#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Given a DRX file, plot the time averaged spectra for each beam output."""

import sys
import math
import numpy
import ephem
import argparse

import lsl.correlator.fx as fxc
from lsl.reader.ldp import LWA1DataFile
from lsl.astro import unix_to_utcjd, DJD_OFFSET
from lsl.misc import parser as aph

import matplotlib.pyplot as plt


def bestFreqUnits(freq):
    """Given a numpy array of frequencies in Hz, return a new array with the
    frequencies in the best units possible (kHz, MHz, etc.)."""
    
    # Figure out how large the data are
    scale = int(math.log10(freq.max()))
    if scale >= 9:
        divis = 1e9
        units = 'GHz'
    elif scale >= 6:
        divis = 1e6
        units = 'MHz'
    elif scale >= 3:
        divis = 1e3
        units = 'kHz'
    else:
        divis = 1
        units = 'Hz'
        
    # Convert the frequency
    newFreq = freq / divis
    
    # Return units and freq
    return (newFreq, units)


def main(args):
    # Length of the FFT
    LFFT = args.fft_length
    
    idf = LWA1DataFile(args.filename)
    
    nFramesFile = idf.getInfo('nFrames')
    srate = idf.getInfo('sampleRate')
    beampols = idf.getInfo('beampols')
    
    # Offset in frames for beampols beam/tuning/pol. sets
    args.skip = idf.offset(args.skip)
    
    # Make sure that the file chunk size contains is an integer multiple
    # of the FFT length so that no data gets dropped.  This needs to
    # take into account the number of beampols in the data, the FFT length,
    # and the number of samples per frame.
    maxFrames = int(1.0*19144/beampols*4096/float(LFFT))*LFFT/4096*beampols
    
    # Number of frames to integrate over
    nFrames = int(args.average * srate / 4096 * beampols)
    nFrames = int(1.0 * nFrames / beampols*4096/float(LFFT))*LFFT/4096*beampols
    args.average = 1.0 * nFrames / beampols * 4096 / srate
    
    # Number of remaining chunks
    nChunks = int(math.ceil(1.0*(nFrames)/maxFrames))
    
    # Date & Central Frequnecy
    beginDate = ephem.Date(unix_to_utcjd(idf.getInfo('tStart')) - DJD_OFFSET)
    centralFreq1 = idf.getInfo('freq1')
    centralFreq2 = idf.getInfo('freq2')
    beam = idf.getInfo('beam')
    
    # File summary
    print "Filename: %s" % args.filename
    print "Date of First Frame: %s" % str(beginDate)
    print "Beam: %i" % beam
    print "Tune/Pols: %i" % beampols
    print "Sample Rate: %i Hz" % srate
    print "Tuning Frequency: %.3f Hz (1); %.3f Hz (2)" % (centralFreq1, centralFreq2)
    print "Frames: %i (%.3f s)" % (nFramesFile, 1.0 * nFramesFile / beampols * 4096 / srate)
    print "---"
    print "Offset: %.3f s (%i frames)" % (args.skip, args.skip*srate*beampols/4096)
    print "Integration: %.3f s (%i frames; %i frames per beam/tune/pol)" % (args.average, nFrames, nFrames / beampols)
    print "Chunks: %i" % nChunks
    
    # Sanity check
    if args.skip*srate*beampols/4096 > nFramesFile:
        raise RuntimeError("Requested offset is greater than file length")
    if nFrames > (nFramesFile - args.skip*srate*beampols/4096):
        raise RuntimeError("Requested integration time+offset is greater than file length")
        
    # Setup the window function to use
    if args.bartlett:
        window = numpy.bartlett
    elif args.blackman:
        window = numpy.blackman
    elif args.hanning:
        window = numpy.hanning
    else:
        window = fxc.noWindow
        
    # Master loop over all of the file chunks
    standMapper = [4*(beam-1) + i for i in xrange(4)]
    masterWeight = numpy.zeros((nChunks, 4, LFFT))
    masterSpectra = numpy.zeros((nChunks, 4, LFFT))
    for i in range(nChunks):
        print "Working on chunk #%i of %i" % (i+1, nChunks)
        
        try:
            readT, t, data = idf.read(args.average/nChunks)
        except Exception, e:
            print "Error: %s" % str(e)
            continue
            
        # Calculate the spectra for this block of data and then weight the results by 
        # the total number of frames read.  This is needed to keep the averages correct.
        freq, tempSpec = fxc.SpecMaster(data, LFFT=LFFT, window=window, verbose=args.verbose, SampleRate=srate, ClipLevel=0)
        for stand in xrange(tempSpec.shape[0]):
            masterSpectra[i,stand,:] = tempSpec[stand,:]
            masterWeight[i,stand,:] = int(readT*srate/LFFT)
            
        # We don't really need the data array anymore, so delete it
        del(data)
        
    # Now that we have read through all of the chunks, perform the final averaging by
    # dividing by all of the chunks
    spec = numpy.squeeze( (masterWeight*masterSpectra).sum(axis=0) / masterWeight.sum(axis=0) )
    
    # Frequencies
    freq1 = freq + centralFreq1
    freq2 = freq + centralFreq2
    
    # The plots:  This is setup for the current configuration of 20 beampols
    fig = plt.figure()
    figsX = int(round(math.sqrt(4)))
    figsY = 4 / figsX
    # Put the frequencies in the best units possible
    freq1, units1 = bestFreqUnits(freq1)
    freq2, units2 = bestFreqUnits(freq2)
    
    sortedMapper = sorted(standMapper)
    for k, aStand in enumerate(sortedMapper):
        i = standMapper.index(aStand)
        if standMapper[i]%4/2+1 == 1:
            freq = freq1
            units = units1
        else:
            freq = freq2
            units = units2
            
        ax = fig.add_subplot(figsX,figsY,k+1)
        currSpectra = numpy.squeeze( numpy.log10(spec[i,:])*10.0 )
        ax.plot(freq, currSpectra, label='%i (avg)' % (i+1))
        
        # If there is more than one chunk, plot the difference between the global 
        # average and each chunk
        if nChunks > 1 and not args.disable_chunks:
            for j in range(nChunks):
                # Some files are padded by zeros at the end and, thus, carry no 
                # weight in the average spectra.  Skip over those.
                if masterWeight[j,i,:].sum() == 0:
                    continue
                    
                # Calculate the difference between the spectra and plot
                subspectra = numpy.squeeze( numpy.log10(masterSpectra[j,i,:])*10.0 )
                diff = subspectra - currSpectra
                ax.plot(freq, diff, label='%i' % j)
                
        ax.set_title('Beam %i, Tune. %i, Pol. %i' % (standMapper[i]/4+1, standMapper[i]%4/2+1, standMapper[i]%2))
        ax.set_xlabel('Frequency [%s]' % units)
        ax.set_ylabel('P.S.D. [dB/RBW]')
        ax.set_xlim([freq.min(), freq.max()])
        ax.legend(loc=0)
        
        print "For beam %i, tune. %i, pol. %i maximum in PSD at %.3f %s" % (standMapper[i]/4+1, standMapper[i]%4/2+1, standMapper[i]%2, freq[numpy.where( spec[i,:] == spec[i,:].max() )][0], units)
        
    print "RBW: %.4f %s" % ((freq[1]-freq[0]), units)
    plt.subplots_adjust(hspace=0.35, wspace=0.30)
    plt.show()
    
    # Save spectra image if requested
    if args.output is not None:
        fig.savefig(args.output)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
            description='read in a DRX file and create a collection of time-averaged spectra', 
            formatter_class=argparse.ArgumentDefaultsHelpFormatter
            )
    parser.add_argument('filename', type=str, 
                        help='filename to process')
    wgroup = parser.add_mutually_exclusive_group(required=False)
    wgroup.add_argument('-t', '--bartlett', action='store_true', 
                        help='apply a Bartlett window to the data')
    wgroup.add_argument('-b', '--blackman', action='store_true', 
                        help='apply a Blackman window to the data')
    wgroup.add_argument('-n', '--hanning', action='store_true', 
                        help='apply a Hanning window to the data')
    parser.add_argument('-s', '--skip', type=aph.positive_or_zero_float, default=0.0, 
                        help='skip the specified number of seconds at the beginning of the file')
    parser.add_argument('-a', '--average', type=aph.positive_float, default=10.0, 
                        help='number of seconds of data to average for spectra')
    parser.add_argument('-q', '--quiet', dest='verbose', action='store_false',
                        help='run %(prog)s in silent mode')
    parser.add_argument('-l', '--fft-length', type=aph.positive_int, default=4096, 
                        help='set FFT length')
    parser.add_argument('-d', '--disable-chunks', action='store_true', 
                        help='disable plotting chunks in addition to the global average')
    parser.add_argument('-o', '--output', type=str, 
                        help='output file name for spectra image')
    args = parser.parse_args()
    main(args)



# -*- coding: utf-8 -*

"""lsl.writer - Writers for exporting LWA data to various file formats.  
Formats include:
 * tsfits  - FITS files storing time series TBW and TBN data with one extension
             per stand/polarization, 
 * sdfits  - SDFITS filtes for storing single-dish data such as DRX, 
 * fitsidi - FITS IDI writer for storeing correlator output, and
 * vdif    - write data to the VLBI Data Interchange Format."""






# -*- coding: utf-8 -*

"""
lsl.common - Common information for the LSL package.  Including:
 * paths    - paths to the module and its data directories, 
 * stations - information about LWA stations, 
 * dp       - information about the DP system, 
 * mcs      - information about DP-compatible stations at large, 
 * sdf      - read in and interpret SDF files, 
 * adp      - information about the ADP system,
 * mcsADP   - information about ADP-compatible stations at large, and
 * sdfADP   - read in and interpret ADP-compatible SDF files.
"""





#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import sys
import math
import time
import argparse
from datetime import datetime

from lsl.reader import tbn
from lsl.common.progress import ProgressBar
from lsl.misc import parser as aph


def fileSplitFunction(fhIn, fhOut, nCaptures, nAntpols):
    pb = ProgressBar(max=nCaptures)
    
    for c in xrange(int(nCaptures)):
        for i in xrange(nAntpols):
            cFrame = fhIn.read(tbn.FrameSize)
            fhOut.write(cFrame)
            
        pb.inc(amount=1)
        if c != 0 and c % 100 == 0:
            sys.stdout.write(pb.show()+'\r')
            sys.stdout.flush()
            
    sys.stdout.write(pb.show()+'\r')
    sys.stdout.write('\n')
    sys.stdout.flush()


def main(args):
    filename = args.filename
    
    sizeB = os.path.getsize(filename)
    
    # Open the file and get some basic info about the data contained
    fh = open(filename, 'rb')
    sampleRate = tbn.getSampleRate(fh)
    nFramesX, nFramesY = tbn.getFramesPerObs(fh)
    
    nCaptures = sizeB / tbn.FrameSize / (nFramesX + nFramesY)
    
    print "Filename:    %s" % filename
    print "Size:        %.1f MB" % (float(sizeB)/1024/1024)
    print "Captures:    %i (%.2f seconds)" % (nCaptures, nCaptures*512/sampleRate)
    print "Stands:      %i (%i x pol., %i y pol.)" % ((nFramesX+nFramesY), nFramesX, nFramesY)
    print "Sample Rate: %.2f kHz" % (sampleRate/1000.0)
    print "==="

    if args.count > 0:
        nCaptures = args.count * sampleRate / 512
    else:
        nCaptures -= args.offset * sampleRate / 512
        args.count = nCaptures * 512 / sampleRate
    nSkip = int(args.offset * sampleRate / 512)

    print "Seconds to Skip:  %.2f (%i captures)" % (args.offset, nSkip)
    print "Seconds to Split: %.2f (%i captures)" % (args.count, nCaptures)

    # Make sure that the first frame in the file is the first frame if a capture 
    # (stand 1, pol 0).  If not, read in as many frames as necessary to get to 
    # the beginning of a complete capture.
    frame = tbn.readFrame(fh)
    stand, pol = frame.parseID()

    skip = 0
    while (2*(stand-1)+pol) != 0:
        frame = tbn.readFrame(fh)
        stand, pol = frame.parseID()
        skip += 1
    fh.seek(fh.tell() - tbn.FrameSize)

    if skip != 0:
        print "Skipped %i frames at the beginning of the file" % skip
    
    for c in list(range(nSkip)):
        if c < nSkip:
            fh.seek(fh.tell() + tbn.FrameSize*(nFramesX+nFramesY))
            continue
            
    nFramesRemaining = (sizeB - fh.tell()) / tbn.FrameSize
    nRecursions = int(nFramesRemaining / (nCaptures*(nFramesX+nFramesY)))
    if not args.recursive:
        nRecursions = 1
        
    scale = int(math.log10(nRecursions)) + 1
    ifString = "Working on #%%%ii of %i (%%s)" % (scale, nRecursions)
    
    for r in xrange(nRecursions):
        if args.date:
            filePos = fh.tell()
            junkFrame = tbn.readFrame(fh)
            fh.seek(filePos)
            
            dt = datetime.utcfromtimestamp(junkFrame.getTime())
            captFilename = "%s_%s.dat" % (os.path.splitext(os.path.basename(filename))[0], dt.isoformat())
        else:
            captFilename = "%s_s%04i_p%%0%ii.dat" % (os.path.splitext(os.path.basename(filename))[0], args.count, scale)
            captFilename = captFilename % r
            if not args.recursive:
                captFilename = "%s_s%04i.dat" % (os.path.splitext(os.path.basename(filename))[0], args.count)
            
        print ifString % (r+1, captFilename)
        
        t0 = time.time()
        fhOut = open(captFilename, 'wb')
        fileSplitFunction(fh, fhOut, nCaptures, nFramesX+nFramesY)
        fhOut.close()
        t1 = time.time()
        print "  Copied %i bytes in %.3f s (%.3f MB/s)" % (os.path.getsize(captFilename), t1-t0, os.path.getsize(captFilename)/1024.0**2/(t1-t0))
    fh.close()


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description='split a TBN file into several files', 
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
        )
    parser.add_argument('filename', type=str, 
                        help='filename to split')
    parser.add_argument('-c', '--count', type=aph.positive_float, default=10.0, 
                        help='number of seconds to keep')
    parser.add_argument('-o', '--offset', type=aph.positive_or_zero_float, default=0.0, 
                        help='number of seconds to skip before splitting')
    parser.add_argument('-d', '--date', action='store_true', 
                        help='label the split files with a date rather than a sequence number')
    parser.add_argument('-r', '--recursive', action='store_true', 
                        help='recursively split the file')
    args = parser.parse_args()
    main(args)
    



# -*- coding: utf-8 -*-

"""
Module that provides argparse-compatible conversion functions for a variety 
of value formats, including:
 * positive integers, 
 * ephem.hours instances, and
 * lists of integers from a comma-separated list.

.. versionadded:: 1.2.4
"""

import re
import ephem
from argparse import ArgumentTypeError
from datetime import datetime

from lsl.common.constants import c
from lsl.common.mcs import datetime2mjdmpm, mjdmpm2datetime

__version__ = '0.1'
__revision__ = '$Rev: 2640 $'
__all__ = ['positive_or_zero_int', 'positive_int', 'positive_or_zero_float', 
           'positive_float', 'frequency', 'frequency_range', 'wavelength', 
           'wavelength_range', 'date', 'mjd', 'time', 'mpm', 'hours', 
           'csv_hours_list', 'degrees', 'csv_degrees_list', 'csv_int_list', 
           'csv_baseline_list', 'csv_hostname_list']


def positive_or_zero_int(string):
    """
    Convert a string to a positive (>=0) integer.
    """
    
    try:
        value = int(string, 10)
    except ValueError:
        msg = "%r is a non-integer value" % string
        raise ArgumentTypeError(msg)
    if value < 0:
        msg = "%r < 0" % string
        raise ArgumentTypeError(msg)
    return value


def positive_int(string):
    """
    Convert a string to a positive (>0) integer.
    """
    
    value = positive_or_zero_int(string)
    if value <= 0:
        msg = "%r <= 0" % string
        raise ArgumentTypeError(msg)
    return value


def positive_or_zero_float(string):
    """
    Convert a string to a positive (>=0.0) float.
    """
    
    try:
        value = float(string)
    except ValueError:
        msg = "%r is a non-float value" % string
        raise ArgumentTypeError(msg)
    if value < 0.0:
        msg = "%r < 0.0" % string
        raise ArgumentTypeError(msg)
    return value


def positive_float(string):
    """
    Convert a string to a positive (>0.0) float.
    """
    
    value = positive_or_zero_float(string)
    if value <= 0.0:
        msg = "%r <= 0.0" % string
        raise ArgumentTypeError(msg)
    return value


# Prefix conversion factors for _prefix_to_scale()
_PREFIX_SCALES = {'y': 1e-24, 'z': 1e-21, 'a': 1e-18, 'f': 1e-15, 'p': 1e-12, 
                  'n': 1e-9,  'u': 1e-6,  'm': 1e-3,  'c': 1e-2,  'd': 1e-1, 
                  '':  1e0,   'da': 1e1,  'h': 1e2,   'k': 1e3,   'M': 1e6, 
                  'G': 1e9,   'T': 1e12,  'P': 1e15,  'E': 1e18,  'Z': 1e21, 
                  'Y': 1e24}


def _prefix_to_scale(prefix):
    """
    Function to convert a metric prefix to a scale for the base unit.
    """
    
    try:
        scale = _PREFIX_SCALES[prefix]
    except KeyError:
        raise ValueError("Unknown prefix '%s'" % prefix)
    return scale


def _quantitiy_to_hz(value, unit):
    """
    Convert a value/pair into a frequency.  If no unit is provided, MHz is 
    assumed.
    """
    
    value = float(value)
    if unit is None:
        unit = 'MHz'
        
    if unit[-8:] == 'Angstrom' or unit[-2:] == 'AA':
        ## Angstroms
        pscale = 1e-10
        value *= pscale
        value = c/value
    elif unit[-1] == 'm':
        ## Some version of meters
        pscale = _prefix_to_scale(unit[:-1])
        value *= pscale
        value = c/value
    elif unit[-2:] == 'Hz':
        ## Some version of frequency
        pscale = _prefix_to_scale(unit[:-2])
        value *= pscale
    return value


def _quantitiy_to_m(value, unit):
    """
    Convert a value/pair into a wavelength.  If no unit is provided, m is 
    assumed.
    """
    
    value = float(value)
    if unit is None:
        unit = 'm'
        
    if unit[-8:] == 'Angstrom' or unit[-2:] == 'AA':
        ## Angstroms
        pscale = 1e-10
        value *= pscale
    elif unit[-1] == 'm':
        ## Some version of meters
        pscale = _prefix_to_scale(unit[:-1])
        value *= pscale
    elif unit[-2:] == 'Hz':
        ## Some version of frequency
        pscale = _prefix_to_scale(unit[:-2])
        value *= pscale
        value = c/value
    return value


# Regular expression for frequency()
_FREQUENCY_RE = re.compile('^(?P<start>\d*(\.\d*)?)\s*(?P<unit1>(([yzafpnumcdhkMGTPEZY]|(da))?Hz)|(([yzafpnumcdhkMGTPEZY]|(da))?m)|(AA)|(Angstrom))?(\~(?P<stop>\d*(\.\d*)?))?\s*(?P<unit2>(([yzafpnumcdhkMGTPEZY]|(da))?Hz)|(([yzafpnumcdhkMGTPEZY]|(da))?m)|(AA)|(Angstrom))?$')


def _frequency_conversion_base(string):
    """
    Convert a frequency to a float Hz value.  This function accepts a variety 
    of string formats:
     * pure float values are intepreted to be in MHz (45.0 -> 45e6)
     * number/unit pairs are allowed so long as they are in:
        * [prefix]m, A, or ang for wavelength and 
        * [prefix]Hz for frequency
     * a 'number~number' is interpretted as a range in MHz
     * a 'number/unit~number/unit' is converted to a range in Hz
    
    .. note::
        For ranges, a two-element list is returned where the first value
        is less than the second.
    """
    
    try:
        value = float(string)*1e6
    except ValueError:
        mtch = _FREQUENCY_RE.match(string)
        if mtch is None:
            msg = "%r cannot be interpretted as a frequency" % string
            raise ArgumentTypeError(msg)
        else:
            start, stop = mtch.group('start'), mtch.group('stop')
            unit1, unit2 = mtch.group('unit1'), mtch.group('unit2')
            if string.find('~') != -1 and stop is None:
                msg = "%r cannot be parsed as a range" % string
                raise ArgumentTypeError(msg)
            if unit1 is None:
                unit1 = unit2
            elif stop is not None and unit2 is None:
                msg = "%r must have units specified for the second value" % string
                raise ArgumentTypeError(msg)
            value = _quantitiy_to_hz(start, unit1)
            if stop is not None:
                value = [value, _quantitiy_to_hz(stop, unit2)]
                if value[1] < value[0]:
                    value.reverse()
    return value


def frequency(string):
    """
    Convert a frequency to a float Hz value.  This function accepts a variety 
    of string formats:
     * pure float values are intepreted to be in MHz (45.0 -> 45e6)
     * number/unit pairs are allowed so long as they are in:
        * [prefix]m, A, or ang for wavelength and 
        * [prefix]Hz for frequency
    """
    
    value = _frequency_conversion_base(string)
    try:
        len(value)
        msg = "%r does not appear to be a single frequency" % string
        raise ArgumentTypeError(msg)
    except TypeError:
        pass
    return value


def frequency_range(string):
    """
    Convert a frequency to a float Hz value.  This function accepts a variety 
    of string formats:
     * a 'number~number' is interpretted as a range in MHz
     * a 'number/unit~number/unit' is converted to a range in Hz
    
    .. note::
        For ranges, a two-element list is returned where the first value
        is less than the second.
    """
    
    value = _frequency_conversion_base(string)
    try:
        len(value)
    except TypeError:
        msg = "%r does not appear to be a frequency range" % string
        raise ArgumentTypeError(msg)
    return value


def _wavelength_conversion_base(string):
    """
    Convert a wavelength to a float m value.  This function accepts a variety 
    of string formats:
     * pure float values are intepreted to be in m (45.0 -> 45.0)
     * number/unit pairs are allowed so long as they are in:
        * [prefix]m, A, or ang for wavelength and 
        * [prefix]Hz for frequency
     * a 'number~number' is interpretted as a range in m
     * a 'number/unit~number/unit' is converted to a range in m
    
    .. note::
        For ranges, a two-element list is returned where the first value
        is less than the second.
    """
    
    try:
        value = float(string)
    except ValueError:
        mtch = _FREQUENCY_RE.match(string)
        if mtch is None:
            msg = "%r cannot be interpretted as a frequency" % string
            raise ArgumentTypeError(msg)
        else:
            start, stop = mtch.group('start'), mtch.group('stop')
            unit1, unit2 = mtch.group('unit1'), mtch.group('unit2')
            if string.find('~') != -1 and stop is None:
                msg = "%r cannot be parsed as a range" % string
                raise ArgumentTypeError(msg)
            if unit1 is None:
                unit1 = unit2
            elif stop is not None and unit2 is None:
                msg = "%r must have units specified for the second value" % string
                raise ArgumentTypeError(msg)
            value = _quantitiy_to_m(start, unit1)
            if stop is not None:
                value = [value, _quantitiy_to_m(stop, unit2)]
                if value[1] < value[0]:
                    value.reverse()
    return value


def wavelength(string):
    """
    Convert a wavelength to a float m value.  This function accepts a variety 
    of string formats:
     * pure float values are intepreted to be in m (45.0 -> 45.0)
     * number/unit pairs are allowed so long as they are in:
        * [prefix]m, A, or ang for wavelength and 
        * [prefix]Hz for frequency
    """
    
    value = _wavelength_conversion_base(string)
    try:
        len(value)
        msg = "%r does not appear to be a single wavelength" % string
        raise ArgumentTypeError(msg)
    except TypeError:
        pass
    return value


def wavelength_range(string):
    """
    Convert a wavelength to a float m value.  This function accepts a variety 
    of string formats:
     * a 'number~number' is interpretted as a range in m
     * a 'number/unit~number/unit' is converted to a range in m
    
    .. note::
        For ranges, a two-element list is returned where the first value
        is less than the second.
    """
    
    value = _wavelength_conversion_base(string)
    try:
        len(value)
    except TypeError:
        msg = "%r does not appear to be a wavelength range" % string
        raise ArgumentTypeError(msg)
    return value


def date(string):
    """
    Convert a data as either a YYYY[-/]MM[-/]DD or MJD string into a 
    YYYY/MM/DD string.
    """
    
    try:
        mjd = int(string, 10)
        dt = mjdmpm2datetime(mjd, 0)
    except ValueError:
        cstring = string.replace('-', '/')
        try:
            dt = datetime.strptime("%s 00:00:00" % cstring, "%Y/%m/%d %H:%M:%S")
        except ValueError:
            msg = "%r cannot be interpretted as an MJD or date string" % string
            raise ArgumentTypeError(msg)
            
    date = dt.strftime('%Y/%m/%d')
    return date


def mjd(string):
    """
    Convert a data as either a YYYY[-/]MM[-/]DD or MJD string into an integer
    MJD.
    """
    
    try:
        mjd = int(string, 10)
    except ValueError:
        cstring = string.replace('-', '/')
        try:
            dt = datetime.strptime("%s 00:00:00" % cstring, "%Y/%m/%d %H:%M:%S")
            mjd, mpm = datetime2mjdmpm(dt)
        except ValueError:
            msg = "%r cannot be interpretted as an MJD or date string" % string
            raise ArgumentTypeError(msg)
            
    return mjd


def time(string):
    """
    Covnert a time as HH:MM:SS[.SSS] or MPM string into a HH:MM:SS.SSSSSS 
    string.
    """
    
    try:
        mpm = int(string, 10)
        if mpm < 0 or mpm > (86400*1000 + 999):
            msg = "%r is out of range for an MPM value"
            raise ArgumentTypeError(msg)
        s, f = mpm/1000, mpm%1000
        h = s / 3600
        m = (s / 60) % 60
        s = s % 60
        stime = "%i:%02i:%02i.%06i" % (h, m, s, f*1000)
    except ValueError:
        try:
            dt = datetime.strptime("2000/1/1 %s" % string, "%Y/%m/%d %H:%M:%S.%f")
        except ValueError:
            try:
                dt = datetime.strptime("2000/1/1 %s" % string, "%Y/%m/%d %H:%M:%S")
            except ValueError:
                msg = "%r cannot be interpretted as a time string" % string
                raise ArgumentTypeError(msg)
        stime = "%i:%02i:%02i.%06i" % (dt.hour, dt.minute, dt.second, dt.microsecond)
    return stime


def mpm(string):
    """
    Covnert a time as HH:MM:SS[.SSS] or MPM string into an MPM integer.
    """
    
    try:
        mpm = int(string, 10)
        if mpm < 0 or mpm > (86400*1000 + 999):
            msg = "%r is out of range for an MPM value"
            raise ArgumentTypeError(msg)
    except ValueError:
        try:
            dt = datetime.strptime("2000/1/1 %s" % string, "%Y/%m/%d %H:%M:%S.%f")
        except ValueError:
            try:
                dt = datetime.strptime("2000/1/1 %s" % string, "%Y/%m/%d %H:%M:%S")
            except ValueError:
                msg = "%r cannot be interpretted as a time string" % string
                raise ArgumentTypeError(msg)
        mjd, mpm = datetime2mjdmpm(dt)
    return mpm


def hours(string):
    """
    Convert a 'HH[:MM[:SS[.SSS]]]' string into an ephem.hours instance.
    """
    
    try:
        value = ephem.hours(string)
    except ValueError as e:
        msg = str(e) % string
        raise ArgumentTypeError(msg)
    return value


def csv_hours_list(string):
    """
    Convert a comma-separated list of 'HH[:MM[:SS.[SSS]]]' string into a list 
    of ephem.hours instances.
    """
    
    string = string.rstrip()
    if string[-1] == ',':
        string = string[:-1]
        
    value = []
    for item in string.split(','):
        value.append( hours(item) )
    return value


def degrees(string):
    """
    Convert a 'sDD[:MM[:SS[.SSS]]]' string into an ephem.degrees instance.
    """
    
    try:
        value = ephem.degrees(string)
    except ValueError as e:
        msg = str(e) % string
        raise ArgumentTypeError(msg)
    return value


def csv_degrees_list(string):
    """
    Convert a comma-separated list of 'sDD[:MM[:SS.[SSS]]]' string into a list 
    of ephem.degrees instances.
    """
    
    string = string.rstrip()
    if string[-1] == ',':
        string = string[:-1]
        
    value = []
    for item in string.split(','):
        value.append( degrees(item) )
    return value


def _int_item_or_range(string):
    if string.find('~') != -1:
        start, stop = string.split('~', 1)
        start, stop = int(start, 10), int(stop, 10)
        value = list(range(start, stop+1))
    else:
        value = [int(string, 10),]
    return value


def csv_int_list(string):
    """
    Convert a comma-separated list of integers into a list of integers.  This 
    function also allows for ranges to be specifed using the '~' character.  
    This formatting converts 'start~stop' to 'range(start, stop+1)'.
    """
    
    if string in ('all', '*'):
        value = 'all'
    elif string in ('none', ''):
        value = 'none'
    else:
        value = []
        for item in string.split(','):
            item = item.strip().rstrip()
            if item == '':
                continue
            try:
                subvalue = _int_item_or_range(item)
            except ValueError:
                msg = "%r contains non-integer values" % string
                raise ArgumentTypeError(msg)
            value.extend(subvalue)
    return value


def csv_baseline_list(string):
    """
    Convert a comma-separated list of baslines pairs into a list of baseline
    pairs.  Baseline pairs are defined as 'antenna1-antenna2' where 'antenna1'
    and 'antenna2' are both integers or ranges denoted by the '~' character.
    """
    
    if string in ('all', '*'):
        value = 'all'
    elif string in ('none', ''):
        value = 'none'
    else:
        value = []
        for item in string.split(','):
            item = item.strip().rstrip()
            if item == '':
                continue
            try:
                ant1, ant2 = item.split('-', 1)
            except ValueError:
                msg = "%s contains non-baseline or non-integer values" % string
                raise ArgumentTypeError(msg)
            try:
                ant1 = _int_item_or_range(ant1)
                ant2 = _int_item_or_range(ant2)
            except ValueError:
                msg = "%r contains non-integer values" % string
                raise ArgumentTypeError(msg)
            for i in ant1:
                for j in ant2:
                    value.append( (i,j) )
    return value


_IPV4_RANGE_RE = re.compile('^(?P<byte1>\d{1,3})(~(?P<byte1e>\d{1,3}))?\.(?P<byte2>\d{1,3})(~(?P<byte2e>\d{1,3}))?\.(?P<byte3>\d{1,3})(~(?P<byte3e>\d{1,3}))?\.(?P<byte4>\d{1,3})(~(?P<byte4e>\d{1,3}))?$')

_HOSTNAME_RE = re.compile('^(?P<hostname>[a-zA-Z\-0-9]*?)$')
_HOSTNAME_RANGE_RE=re.compile('^(?P<hostbase>[a-zA-Z\-]*?)(?P<start>[0-9]+)~(?P<stop>[0-9]+)$')
    

def csv_hostname_list(string):
    """
    Convert a comma-separated list of IPv4 addresses/hostnames into a list 
    IPv4 addresses/hostnames.  This function support indexing with the '~' 
    character so long as:
     * the character is in any one of the IPv4 bytes or
     * the character is at the end of a hostname which ends with a number
    """
    
    value = []
    for item in string.split(','):
        item = item.strip().rstrip()
        if item == '':
            continue
        mtch = _IPV4_RANGE_RE.match(item)
        if mtch is not None:
            ## IPv4 address or IPv4 address range
            b1b = int(mtch.group('byte1'), 10)
            b1e = mtch.group('byte1e')
            b1e = int(b1e, 10) if b1e is not None else b1b
            
            b2b = int(mtch.group('byte2'), 10)
            b2e = mtch.group('byte2e')
            b2e = int(b2e, 10) if b2e is not None else b2b
            
            b3b = int(mtch.group('byte3'), 10)
            b3e = mtch.group('byte3e')
            b3e = int(b3e, 10) if b3e is not None else b3b
            
            b4b = int(mtch.group('byte4'), 10)
            b4e = mtch.group('byte4e')
            b4e = int(b4e, 10) if b4e is not None else b4b
            
            items = []
            for b1 in range(b1b, b1e+1):
                for b2 in range(b2b, b2e+1):
                    for b3 in range(b3b, b3e+1):
                        for b4 in range(b4b, b4e+1):
                            items.append( '%i.%i.%i.%i' % (b1, b2, b3, b4) )
        else:
            mtch = _HOSTNAME_RANGE_RE.match(item)
            if mtch is not None:
                ## Hostname range
                hostbase = mtch.group('hostbase')
                try:
                    start = int(mtch.group('start'), 10)
                    stop = int(mtch.group('stop'), 10)
                except ValueError:
                    msg = "%r contains non-integer hostname values" % string
                    raise ArgumentTypeError(msg)
                items = ['%s%i' % (hostbase, i) for i in range(start, stop+1)]
            else:
                mtch = _HOSTNAME_RE.match(item)
                if mtch is not None:
                    ## Single hostname
                    items = [mtch.group('hostname'),]
                else:
                    msg = "%r contains invalid hostname values" % string
                    raise ArgumentTypeError(msg)
        value.extend( items )
    return value



# -*- coding: utf-8 -*-

"""Unit test for the lsl.writer.uvfits modules."""

import os
import time
import unittest
import tempfile
import numpy
import pyfits

from lsl.common import stations as lwa_common
from lsl.correlator import uvUtils
from lsl.writer import uvfits


__version__  = "0.2"
__revision__ = "$Rev: 2675 $"
__author__   = "Jayce Dowell"


class uvfits_tests(unittest.TestCase):
    """A unittest.TestCase collection of unit tests for the lsl.writer.uvfits.uv
    class."""
    
    testPath = None
    
    def setUp(self):
        """Turn off all numpy warnings and create the temporary file directory."""
        
        numpy.seterr(all='ignore')
        self.testPath = tempfile.mkdtemp(prefix='test-uvfits-', suffix='.tmp')
        
    def __initData(self):
        """Private function to generate a random set of data for writing a UVFITS
        file.  The data is returned as a dictionary with keys:
        * freq - frequency array in Hz
        * site - lwa.common.stations object
        * stands - array of stand numbers
        * bl - list of baseline pairs in real stand numbers
        * vis - array of visibility data in baseline x freq format
        """

        # Frequency range
        freq = numpy.arange(0,512)*20e6/512 + 40e6
        # Site and stands
        site = lwa_common.lwa1
        antennas = site.getAntennas()[0:40:2]
        
        # Set baselines and data
        blList = uvUtils.getBaselines(antennas, IncludeAuto=True, Indicies=False)
        visData = numpy.random.rand(len(blList), len(freq))
        visData = visData.astype(numpy.complex64)
        
        return {'freq': freq, 'site': site, 'antennas': antennas, 'bl': blList, 'vis': visData}

    def test_write_tables(self):
        """Test if the UVFITS writer writes all of the tables."""
        
        testTime = time.time()
        testFile = os.path.join(self.testPath, 'uv-test-W.fits')
        
        # Get some data
        data = self.__initData()
        
        # Start the file
        fits = uvfits.UV(testFile, refTime=testTime)
        fits.setStokes(['xx'])
        fits.setFrequency(data['freq'])
        fits.setGeometry(data['site'], data['antennas'])
        fits.addComment('This is a comment')
        fits.addHistory('This is history')
        fits.addDataSet(testTime, 6.0, data['bl'], data['vis'])
        fits.write()
        
        # Open the file and examine
        hdulist = pyfits.open(testFile)
        # Check that all of the extensions are there
        extNames = [hdu.name for hdu in hdulist]
        for ext in ['AIPS AN']:
            self.assertTrue(ext in extNames)
        # Check the comments and history
        self.assertTrue('This is a comment' in str(hdulist[0].header['COMMENT']).split('\n'))
        self.assertTrue('This is history' in str(hdulist[0].header['HISTORY']).split('\n'))
            
        hdulist.close()
        
    def test_array_geometry(self):
        """Test the 'AIPS AN' table, part 1."""
        
        testTime = time.time()
        testFile = os.path.join(self.testPath, 'uv-test-AG.fits')
        
        # Get some data
        data = self.__initData()
        
        # Start the file
        fits = uvfits.UV(testFile, refTime=testTime)
        fits.setStokes(['xx'])
        fits.setFrequency(data['freq'])
        fits.setGeometry(data['site'], data['antennas'])
        fits.addDataSet(testTime, 6.0, data['bl'], data['vis'])
        fits.write()
        
        # Open the file and examine
        hdulist = pyfits.open(testFile)
        ag = hdulist['AIPS AN'].data
        # Correct number of stands
        self.assertEqual(len(data['antennas']), len(ag.field('NOSTA')))
        
        # Correct stand names
        names = ['LWA%03i' % ant.stand.id for ant in data['antennas']]
        for name, anname in zip(names, ag.field('ANNAME')):
            self.assertEqual(name, anname)
            
        hdulist.close()
        
    def test_antenna(self):
        """Test the 'AIPS AN' table, part 2."""
        
        testTime = time.time()
        testFile = os.path.join(self.testPath, 'uv-test-AN.fits')
        
        # Get some data
        data = self.__initData()
        
        # Start the file
        fits = uvfits.UV(testFile, refTime=testTime)
        fits.setStokes(['xx'])
        fits.setFrequency(data['freq'])
        fits.setGeometry(data['site'], data['antennas'])
        fits.addDataSet(testTime, 6.0, data['bl'], data['vis'])
        fits.write()
        
        # Open the file and examine
        hdulist = pyfits.open(testFile)
        an = hdulist['AIPS AN'].data
        # Correct number of stands
        self.assertEqual(len(data['antennas']), len(an.field('NOSTA')))
        
        hdulist.close()
        
    def test_uvdata(self):
        """Test the primary data table."""
        
        testTime = time.time()
        testFile = os.path.join(self.testPath, 'uv-test-UV.fits')
        
        # Get some data
        data = self.__initData()
        
        # Start the file
        fits = uvfits.UV(testFile, refTime=testTime)
        fits.setStokes(['xx'])
        fits.setFrequency(data['freq'])
        fits.setGeometry(data['site'], data['antennas'])
        fits.addDataSet(testTime, 6.0, data['bl'], data['vis'])
        fits.write()
        
        # Open the file and examine
        hdulist = pyfits.open(testFile)
        uv = hdulist[0]
        
        # Load the mapper
        try:
            mp = hdulist['NOSTA_MAPPER'].data
            nosta = mp.field('NOSTA')
            noact = mp.field('NOACT')
        except KeyError:
            ag = hdulist['AIPS AN'].data
            nosta = ag.field('NOSTA')
            noact = ag.field('NOSTA')
        mapper = {}
        for s,a in zip(nosta, noact):
            mapper[s] = a
            
        # Correct number of visibilities
        self.assertEqual(len(uv.data), data['vis'].shape[0])
        
        # Correct number of frequencies
        for row in uv.data:
            vis = row['DATA'][0,0,:,:,:]
            self.assertEqual(len(vis), len(data['freq']))
            
        # Correct values
        for row in uv.data:
            bl = int(row['BASELINE'])
            vis = row['DATA'][0,0,:,:,:]
            
            # Unpack the baseline
            if bl >= 65536:
                a1 = int((bl - 65536) / 2048)
                a2 = int((bl - 65536) % 2048)
            else:
                a1 = int(bl / 256)
                a2 = int(bl % 256)
            
            # Convert mapped stands to real stands
            stand1 = mapper[a1]
            stand2 = mapper[a2]
            
            # Find out which visibility set in the random data corresponds to the 
            # current visibility
            i = 0
            for ant1,ant2 in data['bl']:
                if ant1.stand.id == stand1 and ant2.stand.id == stand2:
                    break
                else:
                    i = i + 1
                    
            # Extract the data and run the comparison
            visData = numpy.zeros(len(data['freq']), dtype=numpy.complex64)
            visData.real = vis[:,0,0]
            visData.imag = vis[:,0,1]
            for vd, sd in zip(visData, data['vis'][i,:]):
                self.assertAlmostEqual(vd, sd, 8)
            i = i + 1
            
        hdulist.close()
        
    def tearDown(self):
        """Remove the test path directory and its contents"""
        
        tempFiles = os.listdir(self.testPath)
        for tempFile in tempFiles:
            os.unlink(os.path.join(self.testPath, tempFile))
        os.rmdir(self.testPath)
        self.testPath = None


class uvfits_test_suite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the lsl.reader units 
    tests."""
    
    def __init__(self):
        unittest.TestSuite.__init__(self)
        
        loader = unittest.TestLoader()
        self.addTests(loader.loadTestsFromTestCase(uvfits_tests))


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

"""
LWA astronomical source catalogs.
"""


import os
import math
import abc
import collections
import logging

from lsl import astro
from lsl import transform
from lsl.common.paths import data as dataPath

__version__   = '0.2'
__revision__ = '$Rev: 2383 $'
__author__    = 'D. L. Wood'
__maintainer__ = 'Jayce Dowell'


class CatalogEntry(object):
    """
    Represents one source entry in a catalogue.
    
    Contains members:
    * name        - The source name.
    * position    - The source equatorial J2000 position as object
        of type transform.CelestialPosition.
    * alias_list  - A list of strings providing alternate names for
        the source.
    """
    
    # limit the class attributes
    __slots__ = ('name', 'position', 'alias_list')
    
    def __init__(self, name, position):
        """
        Create a catalog entry.
        """
        
        self.name = name
        self.position = position
        self.alias_list = []
        
    def __repr__(self):
        """
        Low level string representation.
        """
        
        return "%s.%s(%s,%s)" % (type(self).__module__, type(self).__name__, repr(self.name), repr(self.position))


class Catalog(collections.Mapping):
    """
    Class representing astronomical source catalog information.
    This is an abstract class; derived classes must provide a
    parse_file() method which populates the catalog object with
    information from file or other source.
    
    Catalog instances support the read-only collections.Mapping
    interface.  That is, they support the read-only methods of
    the dict built-in type.
    """
    
    __metaclass__ = abc.ABCMeta
    
    # a logging instance for catalogs
    log = logging.getLogger('catalog')
    
    def __init__(self, name):
        """
        Create a source catalog.
        """
        
        # initialize catalog data structures
        self.name = name
        self.source_map = {}
        self.alias_map = {}
        
        # parse_file() is an abstract method which must be defined in
        # a concrete implementation for a particular catalog
        self.log.debug("parsing data file for catalog %s", name)    
        self.parse_file()
        
    @abc.abstractmethod
    def parse_file(self):
        """
        Read catalog information from file into internal data
        structures.
        """
        
        pass
        
    @staticmethod
    def get_directory():
        """
        Returns the path to the catalog data file directory.
        """
            
        return os.path.join(dataPath, 'catalog')
        
    def __repr__(self):   
        """
        Low level string representation.
        """
            
        return "%s.Catalog(%s)" % (type(self).__module__, repr(self.name))
        
    def __len__(self):
        """
        Return the number of sources in the catalog.
        """
        
        return len(self.source_map)
        
    def __getitem__(self, key):
        """
        Access source by subscript name.  Raises KeyError if the source
        is not in the catalog.
        """
        
        entry = self.lookup(key)
        if entry is None:
            raise KeyError("name %s not in catalog" % repr(key))
        return entry
        
    def __iter__(self):
        """
        Return an iterator to the primary names in the catalog.
        """
        
        return iter(self.source_map.keys())
        
    def lookup(self, name):
        """
        Lookup a source in the catalog.
        
        Param: name - The primary name or alias of the source.
        
        Returns: An object of type CatalogEntry giving the source information,
                or None if the name is not found in the catalog.
        """
        
        try:
            entry = self.source_map[name]
        except KeyError:
            try:
                entry = self.alias_map[name]
            except KeyError:
                entry = None
                
        return entry


class LWA_Catalog(Catalog):
    """
    Specific definition for LWA observation source catalogue data file.
    """         
    
    def __init__(self):
        """
        Create a LWA catalog instance.
        """
        
        Catalog.__init__(self, 'LWA')
        
    def parse_file(self):
        """
        Read a source catalog data file.
        """
        
        # open data file 
        fileName = os.path.join(self.get_directory(), 'lwa_catalog.dat')
        with open(fileName, 'r') as catFile:
            # read source info
            lineNum = 0
            for line in catFile:
                lineNum += 1
                if line.startswith('#') or line.isspace():
                    continue
                    
                try:
                    name = line[0:8]
                    raHours = int(line[9:11], 10)
                    raMinutes = int(line[12:14], 10)
                    raSeconds = float(line[15:20])
                    decSign = line[21]
                    decDegrees = int(line[22:24], 10)
                    decMinutes = int(line[25:27], 10)
                    decSeconds = float(line[28:32])
                except ValueError as err:
                    raise RuntimeError("file %s, line %d incorectly formated: \'%s\' : %s]" % (fileName, lineNum, line, err))
                    
                name = name.rstrip()           
                
                ra = astro.hms(raHours, raMinutes, raSeconds)
                if decSign == '-':
                    sign = True
                else:
                    sign = False
                dec = astro.dms(sign, decDegrees, decMinutes, decSeconds)
                
                entry = CatalogEntry(name, transform.CelestialPosition((ra, dec), name=name))
                self.source_map[name] = entry
                
                aliasList = line[34:].split()
                if len(aliasList) > 0:
                    for alias in aliasList:
                        entry.alias_list.append(alias)
                        self.alias_map[alias] = entry


class PSR_Catalog(Catalog):
    """
    Specific definition for ATNF Pulsar (PSRCAT) catalog.
    Data file is psrcat.db which can be retreived from:
    <http://www.atnf.csiro.au/research/pulsar/psrcat/download.html>
    """ 
    
    def __init__(self):
        """
        Create an instance of the PSR catalog.
        """
        
        Catalog.__init__(self, 'PSR')
        
    def parse_file(self):
        """
        Read a source catalog data file.
        """
        
        debug = False
        # open data file 
        fileName = os.path.join(self.get_directory(), 'psrcat.db')
        with open(fileName, 'r') as catFile:
            # read source info
            psrb = None
            psrj = None
            ra = None
            dec = None
            bad = False
            for line in catFile:
                if line.startswith('PSRB'):
                    psrb = line.split()[1]
                if line.startswith('PSRJ'):
                    psrj = line.split()[1]
                if line.startswith('RAJ'):
                    rastr = line.split()[1]
                    sp = rastr.split(':')
                    if len(sp) == 3:
                        (raHours, raMinutes, raSeconds) = sp
                    elif len(sp) == 2:
                        (raHours, raMinutes) = sp
                        raSeconds = 0.0
                    else:
                        if debug:
                            print('Bad format for RAJ line : '+line)
                        bad = True
                    raHours = int(raHours)
                    raMinutes = int(raMinutes)
                    raSeconds = float(raSeconds)
                    try:
                        ra = astro.hms(raHours, raMinutes, raSeconds)
                    except:
                        if debug:
                            print('PSRCAT: Bad RA for ', psrj, " : ", rastr)
                        bad = True
                if line.startswith('DECJ'):
                    decstr = line.split()[1]
                    sp = decstr.split(':')
                    if len(sp) == 3:
                        (decDegrees, decMinutes, decSeconds) = sp
                    elif len(sp) == 2:
                        (decDegrees, decMinutes) = sp
                        decSeconds = 0.0
                    else:
                        if debug:
                            print('PSRCAT: Bad format for DECJ line : '+line)
                        bad = True
                        continue
                    if decDegrees.startswith('-'):
                        decDegrees = decDegrees[1:]
                        sign = True
                    else:
                        sign = False
                    decDegrees = int(decDegrees)
                    decMinutes = int(decMinutes)
                    decSeconds = float(decSeconds)
                    try:
                        dec = astro.dms(sign, decDegrees, decMinutes, decSeconds)
                    except:
                        if debug:
                            print('PSRCAT: Bad DEC for ', psrj, " : ", decstr)
                        bad = True
                        
                if line.startswith('@-'):
                    # New source, save current source info
                    if psrb is not None:
                        name = psrb
                        alias = psrj
                    else:
                        name = psrj
                        alias = None
                        
                    if ra is None or dec is None:
                        # These pulsars don't have RAJ, DECJ
                        # I think they may all have ecliptic positions
                        # which should be converted to ra,dec but I'm
                        # going to ignore them for now. -- paulr
                        #print "PSRCAT: No position for pulsar ",name
                        bad = True
                        
                    # Add source to list if good.
                    if not bad:                    
                        sourcePos = astro.equ_posn(ra, dec) 
                        entry = CatalogEntry(name, transform.CelestialPosition(sourcePos, name=name))
                        self.source_map[name] = entry
                        if debug:
                            print('Added ', name)
                        
                        if alias is not None:
                            alias = alias.rstrip()
                            self.alias_map[alias] = entry
                            entry.alias_list.append(alias)
                            if debug:
                                print('Alias : ', alias.rstrip())
                                
                    # Clear out vars for next source
                    psrb = None
                    psrj = None
                    ra = None
                    dec = None
                    bad = False


class PKS_Catalog(Catalog):
    """
    Specific definition for PKS source catalog.
    """
    
    def __init__(self):
        """
        Create a PKS catalog instance.
        """
        
        Catalog.__init__(self, 'PKS')
        
    def parse_file(self):
        """
        Read a source catalog data file.
        """
        
        # open data file 
        fileName = os.path.join(self.get_directory(), 'pkscat.txt')
        with open(fileName, 'r') as catFile:
            # read source info
            lineNum = 1
            line = catFile.readline()
            while line:
                if line == '\n':
                    line = catFile.readline()
                    lineNum += 1
                    continue
                    
                try:
                    name = line[0:8]
                    alias = line[12:19]
                    raHours = int(line[22:24])
                    raMinutes = int(line[25:27])
                    raSeconds = float(line[28:32])
                    decSign = line[33]
                    decDegrees = int(line[34:36])
                    decMinutes = int(line[37:39])
                    decSeconds = int(line[40:42])
                except ValueError:
                    raise RuntimeError("file %s, line %d incorectly formated [%s]" % (fileName, lineNum, line))
                    
                ra = astro.hms(raHours, raMinutes, raSeconds)
                if decSign == '-':
                    sign = True
                else:
                    sign = False
                dec = astro.dms(sign, decDegrees, decMinutes, decSeconds)
                sourcePos = astro.equ_posn(ra, dec)
                
                # precess coordinates from B1950
                entry = CatalogEntry(name, transform.CelestialPosition(sourcePos, epoch='B1950', name=name))
                self.source_map[name] = entry
                
                if len(alias.strip()):
                    alias = alias.rstrip()
                    self.alias_map[alias] = entry
                    entry.alias_list.append(alias)
                    
                line = catFile.readline()
                lineNum += 1


class PKS90_Catalog(Catalog):
    """
    Specific definition for PKS90 source catalogue data file.
    """
    
    def __init__(self):
        """
        Create a PKS90 catalog instance.
        """
        
        Catalog.__init__(self, 'PKS90')
        
    def parse_file(self):
        """
        Read a source catalog data file.
        """
        
        # open data file 
        fileName = os.path.join(self.get_directory(), 'PKS90.txt')
        with open(fileName, 'r') as catFile:
            # read source info
            lineNum = 1
            line = catFile.readline()
            while line:
                if line == '\n':
                    line = catFile.readline()
                    lineNum += 1
                    continue
                    
                try:
                    name = line[0:10]
                    alias0 = line[139:148]
                    alias1 = line[168:175]
                    raHours = int(line[10:12])
                    raMinutes = int(line[13:15])
                    raSeconds = float(line[16:20])
                    decSign = line[23]
                    decDegrees = int(line[24:26])
                    decMinutes = int(line[27:29])
                    decSeconds = float(line[30:34])
                except ValueError:
                    raise RuntimeError("file %s, line %d incorectly formated [%s]" % (fileName, lineNum, line))
                    
                ra = astro.hms(raHours, raMinutes, raSeconds)
                if decSign == '-':
                    sign = True
                else:
                    sign = False
                dec = astro.dms(sign, decDegrees, decMinutes, decSeconds)
                sourcePos = astro.equ_posn(ra, dec)
                
                entry = CatalogEntry(name, transform.CelestialPosition(sourcePos, name=name))
                self.source_map[name] = entry
                
                if len(alias0.strip()): 
                    alias = alias0.rstrip()
                    entry.alias_list.append(alias)
                    self.alias_map[alias] = entry
                if len(alias1.strip()):
                    alias = alias1.rstrip()
                    entry.alias_list.append(alias)
                    self.alias_map[alias] = entry
                    
                line = catFile.readline()
                lineNum += 1


class C3C_Catalog(Catalog):
    """
    Specific definition for Cambridge 3C source catalogue data file.
    """ 
    
    def __init__(self):
        """
        Create a 3C catalog instance.
        """
        
        Catalog.__init__(self, '3C')
        
    def parse_file(self):
        """
        Read a source catalog data file.
        """
        
        # open data file 
        fileName = os.path.join(self.get_directory(), '3c.dat')
        with open(fileName, 'r') as catFile:
            # read source info
            lineNum = 1
            line = catFile.readline()
            while line:
                try:
                    name = line[0:3]
                    raHours = int(line[12:14])
                    raMinutes = int(line[15:17])
                    raSeconds = float(line[18:22])
                    decSign = line[27]
                    decDegrees = int(line[28:30])
                    decMinutes = float(line[31:35])
                except ValueError:
                    raise RuntimeError("file %s, line %d incorectly formated [%s]" % (fileName, lineNum, line))
                    
                name = ('3C' + name.strip())         
                
                ra = astro.hms(raHours, raMinutes, raSeconds)
                (decSeconds, decMinutes) = math.modf(decMinutes)
                if decSign == '-':
                    sign = True
                else:
                    sign = False
                dec = astro.dms(sign, decDegrees, int(decMinutes), (60 * decSeconds))
                sourcePos = astro.equ_posn(ra, dec)
                
                # precess coordinates from B1950
                entry = CatalogEntry(name, transform.CelestialPosition(sourcePos, epoch='B1950', name=name))
                self.source_map[name] = entry        
                
                line = catFile.readline()
                lineNum += 1      


class C4C_Catalog(Catalog):
    """
    Specific definition for Cambridge 4C source catalogue data file.
    """ 
    
    def __init__(self):
        """
        Create a 4C catalog instance.
        """
        
        Catalog.__init__(self, '4C')
        
    def parse_file(self):
        """
        Read a source catalog data file.
        """
        
        # open data file 
        fileName = os.path.join(self.get_directory(), '4c.dat')
        with open(fileName, 'r') as catFile:
            # read source info
            lineNum = 1
            line = catFile.readline()
            while line:
                try:
                    name = line[0:8]
                    raHours = int(line[10:12])
                    raMinutes = int(line[13:15])
                    raSeconds = float(line[16:20])
                    decSign = line[22]
                    decDegrees = int(line[23:25])
                    decMinutes = float(line[26:30])
                    alias = line[64:-1]
                except ValueError:
                    raise RuntimeError("file %s, line %d incorectly formated [%s]" % (fileName, lineNum, line))
                    
                name = name.strip()        
                name = ('4C' + name)
                
                alias = alias.strip()
                alias = alias.rstrip('*')         
                
                ra = astro.hms(raHours, raMinutes, raSeconds)
                (decSeconds, decMinutes) = math.modf(decMinutes)
                if decSign == '-':
                    sign = True
                else:
                    sign = False
                dec = astro.dms(sign, decDegrees, int(decMinutes), (60 * decSeconds))
                sourcePos = astro.equ_posn(ra, dec)
                
                # precess coordinates from B1950
                entry = CatalogEntry(name, transform.CelestialPosition(sourcePos, epoch='B1950', name=name))
                self.source_map[name] = entry
                
                if len(alias.strip()):
                    alias = alias.rstrip()
                    entry.alias_list.append(alias)
                    self.alias_map[alias] = entry       
                    
                line = catFile.readline()
                lineNum += 1      


class F2FGL_Catalog(Catalog):
    """
    Specific definition for Fermi LAT 2-year point source catalog.
    """
    
    def __init__(self):
        """
        Create a 2FGL catalog instance.
        """
        
        Catalog.__init__(self, '2FGL')
        
    def parse_file(self):
        """
        Read a source catalogue data file.
        """
        
        import pyfits
        
        # open data file
        fileName = os.path.join(self.get_directory(), 'gll_psc_v08.fit')
        catFile = pyfits.open(fileName)
        
        # read source info
        sourceTable = catFile['LAT_POINT_SOURCE_CATALOG'].data
        
        for row in sourceTable:
            name = str(row.field('Source_Name')).replace(' ', '_')
            ra = float(row.field('RAJ2000'))
            dec = float(row.field('DEJ2000'))
            entry = CatalogEntry(name, 
            transform.CelestialPosition((ra, dec), name=name))
            self.source_map[name] = entry
            
            alias = str(row.field('0FGL_Name'))
            if len(alias):
                alias = alias.replace(' ', '_')
                self.alias_map[alias] = entry
                entry.alias_list.append(alias)
                
            alias = str(row.field('1FGL_Name'))
            if len(alias):
                alias = alias.replace(' ', '_')
                self.alias_map[alias] = entry
                entry.alias_list.append(alias)
                
            alias = str(row.field('ASSOC_GAM1'))
            if len(alias):
                alias = alias.replace(' ', '_')
                self.alias_map[alias] = entry
                entry.alias_list.append(alias)
                
            alias = str(row.field('ASSOC_GAM2'))
            if len(alias):
                alias = alias.replace(' ', '_')
                self.alias_map[alias] = entry
                entry.alias_list.append(alias)
                
            alias = str(row.field('ASSOC1'))
            if len(alias):
                alias = alias.replace(' ', '_')
                self.alias_map[alias] = entry
                entry.alias_list.append(alias)
                
            alias = str(row.field('ASSOC2'))
            if len(alias):
                alias = alias.replace(' ', '_')
                self.alias_map[alias] = entry
                entry.alias_list.append(alias)
                
        catFile.close()


class CatalogFactory(object):
    """
    Get catalog objects by name.  Caches the catalog data so that
    the data file is parsed only once per session.
    """
    
    # a mapping of catalog names to classes
    catalog_class_map = \
                    {
                        'LWA'   : LWA_Catalog,
                        'PSR'   : PSR_Catalog,
                        'PKS'   : PKS_Catalog,
                        'PKS90' : PKS90_Catalog,
                        '3C'    : C3C_Catalog,
                        '4C'    : C4C_Catalog,
                        '2FGL'  : F2FGL_Catalog,
                    }
                    
    # a mapping of catalog names to instances
    catalog_instance_map = {}
    
    @classmethod
    def get_catalog(klass, name):
        """
        Returns a Catalog object representing the catalog
        given by name.
        """
        
        # check parameters
        if name not in list(klass.catalog_class_map.keys()):
            raise ValueError("unknown catalog \'%s\'" % name)
            
        # try to find an already created instance
        # if not found, create a new instance and cache
        
        try:
            catalog = klass.catalog_instance_map[name]
        except KeyError:
            catalogClass = klass.catalog_class_map[name]
            catalog = catalogClass()
            klass.catalog_instance_map[name] = catalog
            
        return catalog
        
    @classmethod
    def get_names(klass):
        """
        Return a list of known catalog names.
        """
        
        return list(klass.catalog_class_map.keys())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Example script to read in the positions of stands at LWA-1 and make a plot
of the site."""

import sys
import numpy
import argparse

from lsl.common import stations, metabundle, metabundleADP

import matplotlib.pyplot as plt
from matplotlib.ticker import NullFormatter


def main(args):
    # Parse command line
    toMark = numpy.array(args.stand)-1
    
    # Setup the LWA station information
    if args.metadata is not None:
        try:
            station = stations.parseSSMIF(args.metadata)
        except ValueError:
            try:
                station = metabundle.getStation(args.metadata, ApplySDM=True)
            except:
                station = metabundleADP.getStation(args.metadata, ApplySDM=True)
    elif args.lwasv:
        station = stations.lwasv
    else:
        station = stations.lwa1
    stands = station.getStands()
    stands.sort()

    # Load in the stand position data
    data = numpy.zeros((len(stands)/2,3))
    
    i = 0
    for stand in stands[::2]:
        data[i,0] = stand.x
        data[i,1] = stand.y
        data[i,2] = stand.z
        i += 1
        
    # Color-code the stands by their elevation
    color = data[:,2]
    
    # Plot the stands as colored circles
    fig = plt.figure(figsize=(8,8))
    
    ax1 = plt.axes([0.30, 0.30, 0.60, 0.60])
    ax2 = plt.axes([0.30, 0.05, 0.60, 0.15])
    ax3 = plt.axes([0.05, 0.30, 0.15, 0.60])
    ax4 = plt.axes([0.05, 0.05, 0.15, 0.15])
    c = ax1.scatter(data[:,0], data[:,1], c=color, s=40.0, alpha=0.50)	
    ax1.set_xlabel('$\Delta$X [E-W; m]')
    ax1.set_xlim([-80, 80])
    ax1.set_ylabel('$\Delta$Y [N-S; m]')
    ax1.set_ylim([-80, 80])
    ax1.set_title('%s Site:  %.3f$^\circ$N, %.3f$^\circ$W' % (station.name, station.lat*180.0/numpy.pi, -station.long*180.0/numpy.pi))
    
    ax2.scatter(data[:,0], data[:,2], c=color, s=40.0)
    ax2.xaxis.set_major_formatter( NullFormatter() )
    ax2.set_ylabel('$\Delta$Z [m]')
    ax3.scatter(data[:,2], data[:,1], c=color, s=40.0)
    ax3.yaxis.set_major_formatter( NullFormatter() )
    ax3.set_xlabel('$\Delta$Z [m]')
    
    # Explicitly mark those that need to be marked
    if toMark.size != 0:
        for i in xrange(toMark.size):
            ax1.plot(data[toMark[i],0], data[toMark[i],1], marker='x', linestyle=' ', color='black')
            ax2.plot(data[toMark[i],0], data[toMark[i],2], marker='x', linestyle=' ', color='black')
            ax3.plot(data[toMark[i],2], data[toMark[i],1], marker='x', linestyle=' ', color='black')
            
            if args.label:
                ax1.annotate('%i' % (toMark[i]+1), xy=(data[toMark[i],0], data[toMark[i],1]), xytext=(data[toMark[i],0]+1, data[toMark[i],1]+1))
                
    # Add and elevation colorbar to the right-hand side of the figure
    cb = plt.colorbar(c, cax=ax4, orientation='vertical', ticks=[-2, -1, 0, 1, 2])
    
    # Set the axis limits
    ax1.set_xlim([-60, 60])
    ax1.set_ylim([-60, 60])
    ax2.set_xlim( ax1.get_xlim() )
    ax3.set_ylim( ax1.get_ylim() )
    
    # Show n' save
    plt.show()
    if args.output is not None:
        fig.savefig(args.output)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description='plot the x, y, and z locations of stands at an LWA station', 
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
        )
    parser.add_argument('stand', type=int, nargs='*', 
                        help='stand number to mark')
    parser.add_argument('-s', '--lwasv', action='store_true', 
                        help='use LWA-SV instead of LWA1')
    parser.add_argument('-m', '--metadata', type=str, 
                        help='name of the SSMIF or metadata tarball file to use for mappings')
    parser.add_argument('-l', '--label', action='store_true', 
                        help='label the specified stands with their ID numbers')
    parser.add_argument('-v', '--verbose', action='store_true', 
                        help='run %(prog)s in verbose mode')
    parser.add_argument('-o', '--output', type=str, 
                        help='filename to save the plot to')
    args = parser.parse_args()
    main(args)
    



#! /usr/bin/env python
# -*- coding: utf-8 -*-

"""Simple LWA1 astronomical source catalogue display application."""

import math
import argparse
import Tkinter

from lsl import astro
from lsl import transform
from lsl import catalog
from lsl.common import stations


__revision__  = "$Revision: 96 $"
__version__   = "0.1"
__author__	= "D.L.Wood"
__maintainer__ = "Jayce Dowell"


class CatalogViewer(object):
    
    def __init__(self, root, args):
        
        self.root = root
        self.period = args.period * 1000
        self.catalog = None
        self.cursor = self.root.cget("cursor")
        site = args.site.lower().replace('-', '')
        if site == 'lwa1':
            station = stations.lwa1
        elif site == 'lwasv':
            station = stations.lwasv
        elif site == 'ovrolwa':
            station = stations.lwa1
            station.lat, station.lon, station.elev = ('37.2397808', '-118.2816819', 1183.4839)
        else:
            raise RuntimeError("Unknown site name: %s" % site)
        self.site = transform.GeographicalPosition((station.long*180.0/math.pi, station.lat*180.0/math.pi), name=station.name)
        
        self.root.title('LWA Catalog Viewer')
        
        frame = Tkinter.Frame(self.root)
        frame.pack(side = Tkinter.TOP, fill = Tkinter.BOTH, expand = True)
        
        self.catalog_name = Tkinter.StringVar()
        frame1 = Tkinter.Frame(frame)
        frame1.pack(side = Tkinter.LEFT, fill = Tkinter.Y)
        catNames = catalog.CatalogFactory.get_names()
        catNames.sort()
        for name in catNames:
            button = Tkinter.Radiobutton(frame1, text = name, value = name, anchor = Tkinter.W,
                variable = self.catalog_name, command = self.select_catalog, indicatoron = False)
            button.pack(fill = Tkinter.X)
        button = Tkinter.Radiobutton(frame1, text = 'PLANETS', value = 'PLANETS', anchor = Tkinter.W,
            variable = self.catalog_name, command = self.select_catalog, indicatoron = False)
        button.pack(fill = Tkinter.X)
        button = Tkinter.Button(frame1, text = 'Select', command = self.select_source)
        button.pack(fill = Tkinter.X, side = Tkinter.BOTTOM)
        
        
        frame2 = Tkinter.Frame(frame)
        frame2.pack(side = Tkinter.RIGHT, fill = Tkinter.BOTH, expand = True)
        yscroll = Tkinter.Scrollbar(frame2, orient = Tkinter.VERTICAL)
        xscroll = Tkinter.Scrollbar(frame2, orient = Tkinter.HORIZONTAL)
        self.source_list = Tkinter.Listbox(frame2, height = 40, width = 60,
            yscrollcommand = yscroll.set, xscrollcommand = xscroll.set, 
            font = "fixed 12 bold", selectmode = Tkinter.SINGLE, exportselection = False,
            selectbackground = 'yellow')
        yscroll.config(command = self.source_list.yview)
        xscroll.config(command = self.source_list.xview)
        self.source_list.bind("<Double-Button-1>", self.select_source)
        yscroll.pack(side = Tkinter.RIGHT, fill = Tkinter.Y)
        xscroll.pack(side = Tkinter.BOTTOM, fill = Tkinter.X)
        self.source_list.pack(side = Tkinter.LEFT, fill = Tkinter.BOTH, expand = True)
        
        frame = Tkinter.Frame(self.root)
        frame.pack(side = Tkinter.BOTTOM, fill = Tkinter.BOTH)
        
        self.utc_time = Tkinter.Label(frame, relief = Tkinter.SUNKEN,
            anchor = Tkinter.W)
        self.last_time = Tkinter.Label(frame, relief = Tkinter.SUNKEN,
            anchor = Tkinter.W)
        self.utc_time.pack(side = Tkinter.LEFT, fill = Tkinter.BOTH, expand = True)
        self.last_time.pack(side = Tkinter.RIGHT, fill = Tkinter.BOTH, expand = True)
        
        self.root.after(0, self.timer)
        
    def timer(self):
    
        currentTime = transform.Time.from_system()
        self.utc_time.config(text = currentTime.utc_str)
        
        last = astro.deg_to_dms(self.site.sidereal(currentTime))
        self.last_time.config(text = "%s LAST: %s" % (self.site.name, last)) 
        
        self.root.after(self.period, self.timer)
        
    def open_catalog(self):
        
        self.source_list.delete(0, Tkinter.END)
        
        if self.catalog_name.get() == 'PLANETS':
        
            self.select_planet()
            self.root.config(cursor = self.cursor)
            return
            
        self.catalog = catalog.CatalogFactory.get_catalog(self.catalog_name.get())
        
        for name in sorted(self.catalog.keys()):
        
            source = self.catalog[name]
            name = source.name.strip().ljust(18)
            s = "%s | " % name
            equ = source.position.j2000_equ
            sra = "%.03f" % equ.ra
            sdec = "%.03f" % equ.dec
            if equ.dec >= 0:
                sdec = '+' + sdec
            equ = equ.format()
            sra = str(equ[0]) + ' (' + sra.zfill(7) + ')'
            sdec = str(equ[1]) + ' (' + sdec.zfill(7) + ') '
            s += "%s | %s " % (sra.rjust(20), sdec.rjust(20))
            for alias in sorted(source.alias_list):
                s += "%s " % alias.strip() 
            self.source_list.insert(Tkinter.END, s)
            
        self.root.config(cursor = self.cursor)
        
    def select_catalog(self):
    
        self.root.config(cursor = "watch")
        self.root.after(0, self.open_catalog)
            
    def select_planet(self):
    
        self.catalog = None
    
        planets = sorted(transform.PlanetaryPosition.known_names)
        for name in planets:
            self.source_list.insert(Tkinter.END, name)
            
    def select_source(self, *args):
        
        select = self.source_list.curselection()
        if not len(select):
            return
        select = self.source_list.get(select[0])
        name = select.split()[0]
        
        SourceWindow(self.catalog, name, self.site, self.period)


class SourceWindow(Tkinter.Toplevel):
    
    def __init__(self, cat, name, site, period):
        
        Tkinter.Toplevel.__init__(self)
        
        self.lift()
        self.catalog = cat
        self.site = site
        self.period = period
        self.name = name
        
        if self.catalog is None:
            catName = 'PLANETS'
        else:
            catName = self.catalog.name
        self.title("%s/%s" % (catName, name))
        
        sourceFrame = Tkinter.Frame(self)
        sourceFrame.pack(expand = True, fill = Tkinter.X)
        
        labelFrame = Tkinter.Frame(sourceFrame)
        labelFrame.pack(side = Tkinter.LEFT, fill = Tkinter.X)
        
        SourceLabel(labelFrame, 'Visible')
        SourceLabel(labelFrame, 'J2000 RA')
        SourceLabel(labelFrame, 'J2000 DEC')
        SourceLabel(labelFrame, 'Current RA')
        SourceLabel(labelFrame, 'Current DEC')
        SourceLabel(labelFrame, 'Galactic Long.')
        SourceLabel(labelFrame, 'Galactic Lat.')
        SourceLabel(labelFrame, 'Azimuth')
        SourceLabel(labelFrame, 'Altitude')
        SourceLabel(labelFrame, 'Current Time')
        SourceLabel(labelFrame, 'Rise Time')
        SourceLabel(labelFrame, 'Transit Time')
        SourceLabel(labelFrame, 'Set Time')
        
        self.values = {}
        
        valueFrame = Tkinter.Frame(sourceFrame)
        valueFrame.pack(side = Tkinter.RIGHT, fill = Tkinter.X, expand = True)
        
        self.values['visible'] = SourceValue(valueFrame)
        self.values['visible'].config(anchor = Tkinter.CENTER, justify = Tkinter.CENTER)
        self.values['mean_ra'] = SourceValue(valueFrame)
        self.values['mean_dec'] = SourceValue(valueFrame)
        self.values['cur_ra'] = SourceValue(valueFrame)
        self.values['cur_dec'] = SourceValue(valueFrame)
        self.values['gal_lng'] = SourceValue(valueFrame)
        self.values['gal_lat'] = SourceValue(valueFrame)
        self.values['az'] = SourceValue(valueFrame)
        self.values['alt'] = SourceValue(valueFrame)
        self.values['date'] = SourceValue(valueFrame)
        self.values['rise'] = SourceValue(valueFrame)
        self.values['transit'] = SourceValue(valueFrame)
        self.values['set'] = SourceValue(valueFrame)		
        
        if self.catalog is None:
            self.position = transform.PlanetaryPosition(self.name)
        else:
            source = self.catalog[self.name]
            self.position = source.position
        self.direction = transform.PointingDirection(self.position, self.site)
        
        self.after(0, self.timer)
        
    def timer(self):
        
        currentTime = transform.Time.from_system()
        
        cur_equ = self.position.apparent_equ(currentTime)
        if self.catalog is None:
            mean_equ = astro.get_equ_prec2(cur_equ, currentTime.utc_jd, astro.J2000_UTC_JD)
        else:
            mean_equ = self.position.j2000_equ
            
        (mean_ra, mean_dec) = mean_equ.format()
        (cur_ra, cur_dec) = cur_equ.format()
        
        if self.catalog is None:
            gal = astro.get_gal_from_equ2000(mean_equ)
        else:
            gal = self.position.j2000_gal
        (gal_lng, gal_lat) = gal.format()
        
        hrz = self.direction.hrz(currentTime)
        rst = self.direction.rst(currentTime)
        if rst is None:
            rise = None
            transit = None
            set = None
        else:
            (rise, set, transit) = rst.format()
            
        if hrz.alt > 10.0:
            self.values['visible'].config(text = "YES", bg = "green")
        else:
            self.values['visible'].config(text = "NO", bg = "red")
            
        self.values['mean_ra'].config(text = mean_ra)
        self.values['mean_dec'].config(text = mean_dec)
        self.values['cur_ra'].config(text = cur_ra)
        self.values['cur_dec'].config(text = cur_dec)
        self.values['gal_lng'].config(text = gal_lng)
        self.values['gal_lat'].config(text = gal_lat)
        self.values['az'].config(text = "%0.3f %s" % (hrz.az, astro.hrz_to_nswe(hrz)))
        self.values['alt'].config(text = "%0.3f" % hrz.alt)
        self.values['date'].config(text = currentTime.utc_str)
        self.values['rise'].config(text = rise)
        self.values['transit'].config(text = transit)
        self.values['set'].config(text = set)
        
        self.after(self.period, self.timer)


class SourceLabel(Tkinter.Label):
    
    def __init__(self, frame, text):
        
        Tkinter.Label.__init__(self, frame, text = text, anchor = Tkinter.W,
            justify = Tkinter.LEFT, relief = Tkinter.GROOVE, font = "fixed 12 bold")
        self.pack(fill = Tkinter.X)


class SourceValue(Tkinter.Label):
    
    def __init__(self, frame):
        
        Tkinter.Label.__init__(self, frame, anchor = Tkinter.E, font = "fixed 12 bold",
            justify = Tkinter.RIGHT, relief = Tkinter.SUNKEN, foreground = 'blue')
        self.pack(fill = Tkinter.X)


if __name__ == '__main__':
    # parse command line
    parser = argparse.ArgumentParser(
        formatter_class=argparse.ArgumentDefaultsHelpFormatter
        )
    parser.add_argument('-s', '--site', type=str, default='LWA-1',
                        help='site name')
    parser.add_argument('-p', '--period', type=int, default=5, 
                        help='update period in seconds')
    args = parser.parse_args()
    
    # create the GUI
    tk = Tkinter.Tk()
    CatalogViewer(tk, args)
    
    # execute the GUI
    try:
        tk.mainloop()
    except KeyboardInterrupt:
        pass
        



# -*- coding: utf-8 -*-

"""Unit test for the lsl.writer.tsfits modules."""

import os
import time
import unittest
import tempfile
import numpy
import pyfits

from lsl.common.paths import dataBuild as dataPath
from lsl.reader import tbw
from lsl.reader import tbn
from lsl.reader import errors
from lsl.writer import tsfits


__revision__ = "$Rev: 2394 $"
__version__  = "0.1"
__author__    = "Jayce Dowell"

tbwFile = os.path.join(dataPath, 'tests', 'tbw-test.dat')
tbnFile = os.path.join(dataPath, 'tests', 'tbn-test.dat')


class tsfits_tests(unittest.TestCase):
    """A unittest.TestCase collection of unit tests for the lsl.writer.tsfits
    module."""

    testPath = None

    def setUp(self):
        """Turn off all numpy warnings and create the temporary file directory."""

        numpy.seterr(all='ignore')
        self.testPath = tempfile.mkdtemp(prefix='test-tsfits-', suffix='.tmp')

    def __getTBW(self):
        """Private function to load in the test TBW data and get the frames."""

        fh = open(tbwFile, 'rb')

        # Frames 1 through 8
        frames = []
        for i in range(1,9):
            frames.append(tbw.readFrame(fh))

        return frames

    def __getTBN(self, vanilla=False):
        """Private function to load in the test TBN data and get the frames.  If 
        the keyword 'vanilla' is set to True, gain, sample rate, and frequency meta-
        data are not added to the frames."""

        fh = open(tbnFile, 'rb')

        # Frames 1 through 8
        frames = []
        for i in range(1,9):
            frames.append(tbn.readFrame(fh))

        if not vanilla:
            # Set some values for the other meta-data
            for frame in frames:
                frame.setSampleRate(100000)

        return frames

    #def test_write_tbw_single(self):
        #"""Test that TBW data can be written to a TS FITS file one at a time."""

        ## Setup the file names
        #testFile = os.path.join(self.testPath, 'tbw-test-WS.fits')

        ## Get some data and organize it into a nice numpy array
        #frames = self.__getTBW()
        #junk = [frame.parseID() for frame in frames]
        #stands = []
        #for j in junk:
            #if j not in stands:
                #stands.append(j)
        #count = {1: 0, 2: 0}
        #frameData = numpy.zeros((len(stands), 4, 2, 400), dtype=numpy.int16)
        #for frame in frames:
            #frameData[frame.parseID()-1, count[frame.parseID()], :, :] = frame.data.xy
            #count[frame.parseID()] = count[frame.parseID()] + 1

        ## Load it in
        #fits = tsfits.TBW(testFile, UseQueue=False)
        #for frame in frames:
            #fits.addStandData(frame)
        #fits.close()

        ## Read it back in
        #hdulist = pyfits.open(testFile)
        ## Check if we have the correct number of extensions
        #self.assertEqual(len(stands)+1, len(hdulist))

        ## Check if the data bits keyword is set correctly
        #self.assertEqual(hdulist[0].header['TBWBITS'], frames[0].getDataBits())

        ## Check each extension for accuracy
        #for hdu in hdulist[1:]:
            #stand = hdu.data
            #sNumb = hdu.header['STAND']
            ## Check the time tags
            #for time in stand.field('time'):
                #self.assertTrue(time % 400 == 0)

            ### Check the pol. values
            ##print stand.field('pol')
            ##for pol in stand.field('pol'):
                ##self.assertTrue(pol in [0, 1], "Polarization value out of range: %i" % pol)

            ## Check the data length
            #x, y = stand.field('data').shape
            #self.assertEqual(x, 8, "Shape mis-match on dimension 1: %i != 8" % x)
            #self.assertEqual(y, 400, "Shape mis-match on dimension 2: %i != 400" % y)

            #for data in stand.field('data'):
                #self.assertEqual(len(data), 400)
            
            ## Check the data itself
            #count = {0: 0, 1: 0}
            #for pol, data in zip(stand.field('pol'), stand.field('data')):
                #for ts, fs in zip(data, frameData[sNumb-1,count[pol],pol,:]):
                    #self.assertEqual(ts, fs)
                #count[pol] = count[pol] + 1
            

        #hdulist.close()

    def test_write_tbw_queue(self):
        """Test that TBW data can be written to a TS FITS file with a queue."""
        
        
        # Setup the file names
        testFile = os.path.join(self.testPath, 'tbw-test-WQ.fits')

        # Get some data and organize it into a nice numpy array
        frames = self.__getTBW()
        junk = [frame.parseID() for frame in frames]
        stands = []
        for j in junk:
            if j not in stands:
                stands.append(j)
        count = {1: 0, 2: 0}
        frameData = numpy.zeros((len(stands), 4, 2, 400), dtype=numpy.int16)
        for frame in frames:
            frameData[frame.parseID()-1, count[frame.parseID()], :, :] = frame.data.xy
            count[frame.parseID()] = count[frame.parseID()] + 1

        # Load it in
        fits = tsfits.TBW(testFile, UseQueue=True)
        for frame in frames:
            fits.addStandData(frame)
        fits.close()

        # Read it back in
        hdulist = pyfits.open(testFile)
        # Check if we have the correct number of extensions
        self.assertEqual(len(stands)+1, len(hdulist))

        # Check if the data bits keyword is set correctly
        self.assertEqual(hdulist[0].header['TBWBITS'], frames[0].getDataBits())

        # Check each extension for accuracy
        for hdu in hdulist[1:]:
            stand = hdu.data
            sNumb = hdu.header['STAND']
            # Check the time tags
            for time in stand.field('time'):
                self.assertTrue(time % 400 == 0)

            # Check the pol. values
            for pol in stand.field('pol'):
                self.assertTrue(pol in [0, 1])

            # Check the data length
            x, y = stand.field('data').shape
            self.assertEqual(x, 8, "Shape mis-match on dimension 1: %i != 8" % x)
            self.assertEqual(y, 400, "Shape mis-match on dimension 2: %i != 400" % y)

            for data in stand.field('data'):
                self.assertEqual(len(data), 400)
            
            # Check the data itself
            count = {0: 0, 1: 0}
            for pol, time, data in zip(stand.field('pol'), stand.field('time'), stand.field('data')):
                for ts, fs in zip(data, frameData[sNumb-1,count[pol],pol,:]):
                    self.assertEqual(ts, fs)
                count[pol] = count[pol] + 1

        hdulist.close()

    def test_write_tbn_singleV(self):
        """Test that TBN data can be written to a TS FITS file one at a time (simple)."""

        # Setup the file names
        testFile = os.path.join(self.testPath, 'tbn-test-WSV.fits')

        # Get some data and organize it into a nice numpy array
        frames = self.__getTBN(vanilla=True)
        junk = [frame.parseID()[0] for frame in frames]
        stands = []
        for j in junk:
            if j not in stands:
                stands.append(j)
        count = {1: 0, 2: 0, 3: 0, 4: 0}
        frameData = numpy.zeros((len(stands), 4, 2, 512), dtype=numpy.singlecomplex)
        for frame in frames:
            stand, pol = frame.parseID()
            frameData[stand-1, count[stand], pol, :] = frame.data.iq
            if pol == 1:
                count[stand] = count[stand] + 1

        # Load it in
        fits = tsfits.TBN(testFile, UseQueue=False)
        for frame in frames:
            fits.addStandData(frame)
        fits.close()

        # Read it back in
        hdulist = pyfits.open(testFile)
        # Check if we have the correct number of extensions
        self.assertEqual(len(stands)+1, len(hdulist))

        for hdu in hdulist[1:]:
            stand = hdu.data
            sNumb = hdu.header['STAND']
            # Check the time tags
            for time in stand.field('time'):
                self.assertTrue(time % 1003520 == 0)

            # Check the pol. values
            for pol in stand.field('pol'):
                self.assertTrue(pol in [0, 1])

            # Check the data length
            x, y = stand.field('data').shape
            self.assertEqual(x, 2, "Shape mis-match on dimension 1: %i != 2" % x)
            self.assertEqual(y, 512, "Shape mis-match on dimension 2: %i != 512" % y)

            for data in stand.field('data'):
                self.assertEqual(data.dtype.kind, 'c')
                self.assertEqual(len(data), 512)
            
            # Check the data itself
            count = {0: 0, 1: 0}
            for pol, data in zip(stand.field('pol'), stand.field('data')):
                for ts, fs in zip(data, frameData[sNumb-1,count[pol],pol,:]):
                    self.assertAlmostEqual(ts, fs, 6)
                count[pol] = count[pol] + 1

        hdulist.close()

    def test_write_tbn_single(self):
        """Test that TBN data can be written to a TS FITS file one at a time (full)."""

        # Setup the file names
        testFile = os.path.join(self.testPath, 'tbn-test-WS.fits')

        # Get some data and organize it into a nice numpy array
        frames = self.__getTBN()
        junk = [frame.parseID()[0] for frame in frames]
        stands = []
        for j in junk:
            if j not in stands:
                stands.append(j)
        count = {1: 0, 2: 0, 3: 0, 4: 0}
        frameData = numpy.zeros((len(stands), 4, 2, 512), dtype=numpy.singlecomplex)
        for frame in frames:
            stand, pol = frame.parseID()
            frameData[stand-1, count[stand], pol, :] = frame.data.iq
            if pol == 1:
                count[stand] = count[stand] + 1

        # Load it in
        fits = tsfits.TBN(testFile, UseQueue=False)
        for frame in frames:
            fits.addStandData(frame)
        fits.close()

        # Read it back in
        hdulist = pyfits.open(testFile)
        # Check if we have the correct number of extensions
        self.assertEqual(len(stands)+1, len(hdulist))

        for hdu in hdulist[1:]:
            stand = hdu.data
            sNumb = hdu.header['STAND']
            # Check the time tags
            for time in stand.field('time'):
                self.assertTrue(time % 1003520 == 0)

            # Check the pol. values
            for pol in stand.field('pol'):
                self.assertTrue(pol in [0, 1])

            # Check the data length
            x, y = stand.field('data').shape
            self.assertEqual(x, 2, "Shape mis-match on dimension 1: %i != 2" % x)
            self.assertEqual(y, 512, "Shape mis-match on dimension 2: %i != 512" % y)

            for data in stand.field('data'):
                self.assertEqual(data.dtype.kind, 'c')
                self.assertEqual(len(data), 512)
            
            # Check the data itself
            count = {0: 0, 1: 0}
            for pol, data in zip(stand.field('pol'), stand.field('data')):
                for ts, fs in zip(data, frameData[sNumb-1,count[pol],pol,:]):
                    self.assertAlmostEqual(ts, fs, 6)
                count[pol] = count[pol] + 1

        hdulist.close()

    def test_write_tbn_queueV(self):
        """Test that TBN data can be written to a TS FITS file with a queue (simple)."""
        
        
        # Setup the file names
        testFile = os.path.join(self.testPath, 'tbn-test-WQV.fits')

        # Get some data
        frames = self.__getTBN(vanilla=True)

        # Load it in
        junk = [frame.parseID()[0] for frame in frames]
        stands = []
        for j in junk:
            if j not in stands:
                stands.append(j)
        count = {1: 0, 2: 0, 3: 0, 4: 0}
        frameData = numpy.zeros((len(stands), 4, 2, 512), dtype=numpy.singlecomplex)
        for frame in frames:
            stand, pol = frame.parseID()
            frameData[stand-1, count[stand], pol, :] = frame.data.iq
            if pol == 1:
                count[stand] = count[stand] + 1

        # Load it in
        fits = tsfits.TBN(testFile, UseQueue=False)
        for frame in frames:
            fits.addStandData(frame)
        fits.close()

        # Read it back in
        hdulist = pyfits.open(testFile)
        # Check if we have the correct number of extensions
        self.assertEqual(len(stands)+1, len(hdulist))

        for hdu in hdulist[1:]:
            stand = hdu.data
            sNumb = hdu.header['STAND']
            # Check the time tags
            for time in stand.field('time'):
                self.assertTrue(time % 1003520 == 0)

            # Check the pol. values
            for pol in stand.field('pol'):
                self.assertTrue(pol in [0, 1])

            # Check the data length
            x, y = stand.field('data').shape
            self.assertEqual(x, 2, "Shape mis-match on dimension 1: %i != 2" % x)
            self.assertEqual(y, 512, "Shape mis-match on dimension 2: %i != 512" % y)

            for data in stand.field('data'):
                self.assertEqual(data.dtype.kind, 'c')
                self.assertEqual(len(data), 512)
            
            # Check the data itself
            count = {0: 0, 1: 0}
            for pol, data in zip(stand.field('pol'), stand.field('data')):
                for ts, fs in zip(data, frameData[sNumb-1,count[pol],pol,:]):
                    self.assertAlmostEqual(ts, fs, 6)
                count[pol] = count[pol] + 1

        hdulist.close()

    def test_write_tbn_queue(self):
        """Test that TBN data can be written to a TS FITS file with a queue (full)."""
        
        # Setup the file names
        testFile = os.path.join(self.testPath, 'tbn-test-WQ.fits')

        # Get some data
        frames = self.__getTBN()

        # Load it in
        junk = [frame.parseID()[0] for frame in frames]
        stands = []
        for j in junk:
            if j not in stands:
                stands.append(j)
        count = {1: 0, 2: 0, 3: 0, 4: 0}
        frameData = numpy.zeros((len(stands), 4, 2, 512), dtype=numpy.singlecomplex)
        for frame in frames:
            stand, pol = frame.parseID()
            frameData[stand-1, count[stand], pol, :] = frame.data.iq
            if pol == 1:
                count[stand] = count[stand] + 1

        # Load it in
        fits = tsfits.TBN(testFile, UseQueue=False)
        for frame in frames:
            fits.addStandData(frame)
        fits.close()

        # Read it back in
        hdulist = pyfits.open(testFile)
        # Check if we have the correct number of extensions
        self.assertEqual(len(stands)+1, len(hdulist))

        for hdu in hdulist[1:]:
            stand = hdu.data
            sNumb = hdu.header['STAND']
            # Check the time tags
            for time in stand.field('time'):
                self.assertTrue(time % 1003520 == 0)

            # Check the pol. values
            for pol in stand.field('pol'):
                self.assertTrue(pol in [0, 1])

            # Check the data length
            x, y = stand.field('data').shape
            self.assertEqual(x, 2, "Shape mis-match on dimension 1: %i != 2" % x)
            self.assertEqual(y, 512, "Shape mis-match on dimension 2: %i != 512" % y)

            for data in stand.field('data'):
                self.assertEqual(data.dtype.kind, 'c')
                self.assertEqual(len(data), 512)
            
            # Check the data itself
            count = {0: 0, 1: 0}
            for pol, data in zip(stand.field('pol'), stand.field('data')):
                for ts, fs in zip(data, frameData[sNumb-1,count[pol],pol,:]):
                    self.assertAlmostEqual(ts, fs, 6)
                count[pol] = count[pol] + 1

        hdulist.close()

    def tearDown(self):
        """Remove the test path directory and its contents"""

        tempFiles = os.listdir(self.testPath)
        for tempFile in tempFiles:
            os.unlink(os.path.join(self.testPath, tempFile))
        os.rmdir(self.testPath)
        self.testPath = None


class tsfits_test_suite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the lsl.reader units 
    tests."""
    
    def __init__(self):
        unittest.TestSuite.__init__(self)
        
        loader = unittest.TestLoader()
        self.addTests(loader.loadTestsFromTestCase(tsfits_tests)) 


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Python3 compatiability
import sys
if sys.version_info > (3,):
    xrange = range
    
"""Unit test for the lsl.correlator.uvUtils module."""

import warnings
import unittest
import numpy

from lsl.correlator import uvUtils
from lsl.common import stations


__version__  = "0.5"
__revision__ = "$Rev: 2470 $"
__author__    = "Jayce Dowell"


class uvUtils_tests(unittest.TestCase):
    """A unittest.TestCase collection of unit tests for the lsl.correlator.uvUtils
    module."""
    
    def setUp(self):
        """Turn off all numpy and python warnings."""

        numpy.seterr(all='ignore')
        warnings.simplefilter('ignore')

    def test_baseline_gen(self):
        """Test that the generated baselines contain the correct numbers of elements."""

        standList = numpy.array([100, 101, 102, 103])

        bl = uvUtils.getBaselines(standList, IncludeAuto=False, Indicies=False)
        self.assertEqual(len(bl), 6)
        bl = uvUtils.getBaselines(standList, IncludeAuto=True, Indicies=False)
        self.assertEqual(len(bl), 10)

    def test_baseline_ind(self):
        """Test that the baselines generated with Indicies do return indicies and vice
        versa."""

        standList = numpy.array([100, 101, 102, 103])

        bl = uvUtils.getBaselines(standList, IncludeAuto=False, Indicies=False)
        bl = numpy.array(bl)
        self.assertTrue(bl.min() == 100)
        bl = uvUtils.getBaselines(standList, IncludeAuto=True, Indicies=False)
        bl = numpy.array(bl)
        self.assertTrue(bl.min() == 100)

        bl = uvUtils.getBaselines(standList, IncludeAuto=False, Indicies=True)
        bl = numpy.array(bl)
        self.assertTrue(bl.max() < 100)
        bl = uvUtils.getBaselines(standList, IncludeAuto=True, Indicies=True)
        bl = numpy.array(bl)
        self.assertTrue(bl.max() < 100)
        
    def test_antenna_lookup(self):
        """Test baseline number to antenna lookup function."""
        
        standList = numpy.array([100, 101, 102, 103])

        bl = uvUtils.getBaselines(standList, IncludeAuto=False, Indicies=False)
        ind = uvUtils.baseline2antenna(0, standList)
        self.assertEqual(ind[0], 100)
        self.assertEqual(ind[1], 101)
        
        ind = uvUtils.baseline2antenna(1, standList, BaselineList=bl)
        self.assertEqual(ind[0], 100)
        self.assertEqual(ind[1], 102)
        
    def test_baseline_lookup(self):
        """Test antennas to baseline lookup function."""
        
        standList = numpy.array([100, 101, 102, 103])
        bl = uvUtils.getBaselines(standList, IncludeAuto=False, Indicies=False)
        
        ind = uvUtils.antenna2baseline(100, 101, standList, IncludeAuto=False, Indicies=False)
        self.assertEqual(ind, 0)
        
        ind = uvUtils.antenna2baseline(100, 102, standList, BaselineList=bl)
        self.assertEqual(ind, 1)
        
        ind = uvUtils.antenna2baseline(0, 3, standList, IncludeAuto=False, Indicies=True)
        self.assertEqual(ind, 2)
        
    def test_computeUVW(self):
        """Test the the computeUVW function runs."""
        
        station = stations.lwa1
        antennas = station.getAntennas()
        
        # Frequency is a scalar
        freq = 45e6
        out = uvUtils.computeUVW(antennas[0:60:2], freq=freq)
        self.assertEqual(len(out.shape), 3)
        self.assertEqual(out.shape[-1], 1)
        
        # Frequency is a list
        freq = [45e6, 60e6]
        out = uvUtils.computeUVW(antennas[0:60:2], freq=freq)
        self.assertEqual(len(out.shape), 3)
        self.assertEqual(out.shape[-1], 2)

        # Frequency is an array
        ## 1-D
        freq = numpy.linspace(45e6, 60e6, 1024)
        out0 = uvUtils.computeUVW(antennas[0:60:2], freq=freq)
        
        ## 2-D
        freq.shape = (512, 2)
        out1 = uvUtils.computeUVW(antennas[0:60:2], freq=freq)
        
        ## 3-D
        freq.shape = (128, 4, 2)
        out2 = uvUtils.computeUVW(antennas[0:60:2], freq=freq)
        
        shape0 = (out0.shape[0], 3, 1024)
        shape1 = (out0.shape[0], 3, 512, 2)
        shape2 = (out0.shape[0], 3, 128, 4, 2)
        
        # Make sure we have the right dimensions
        for i in xrange(len(shape0)):
            self.assertEqual(out0.shape[i], shape0[i])
        for i in xrange(len(shape1)):
            self.assertEqual(out1.shape[i], shape1[i])
        for i in xrange(len(shape2)):
            self.assertEqual(out2.shape[i], shape2[i])
            
        # Make sure the values are the same
        out1.shape = shape0
        out2.shape = shape0
        diff01 = ((out0 - out1)**2).sum()
        diff02 = ((out0 - out2)**2).sum()
        self.assertAlmostEqual(diff01, 0.0, 6)
        self.assertAlmostEqual(diff02, 0.0, 6)
        
    def test_computeUVTrack(self):
        """Test that the computeUVTrack function runs."""
        
        station = stations.lwa1
        antennas = station.getAntennas()
        
        out = uvUtils.computeUVTrack(antennas[0:60:2])
        
        # Make sure we have the right dimensions
        self.assertEqual(out.shape, (435,2,512))
        
        
class uvUtils_test_suite(unittest.TestSuite):
    """A unittest.TestSuite class which contains all of the lsl.reader units 
    tests."""
    
    def __init__(self):
        unittest.TestSuite.__init__(self)
        
        loader = unittest.TestLoader()
        self.addTests(loader.loadTestsFromTestCase(uvUtils_tests)) 


if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

# Python3 compatibility
from __future__ import division

"""
Python module for reading data in from TBN files.This module defines the 
following classes for storing the TBN data found in a file:

Frame
  object that contains all data associated with a particular TBN frame.  
  The primary constituents of each frame are:
    * FrameHeader - the TBN frame header object and
    * FrameData   - the TBN frame data object.
Combined, these two objects contain all of the information found in the 
original TBN frame.

ObservingBlock
object that stores a collection of Frames for all stands/polarizations for 
a particular time.

In addition to storing the data available in the frame, the Frame object also
has attributes for holding information about the gain, central frequency, and
filter code used for the observations.

The functions defined in this module fall into two class:
  1. convert a frame in a file to a Frame object and
  2. describe the format of the data in the file.

For reading in data, use the readFrame function.  It takes a python file-
handle as an input and returns a fully-filled Frame object.  The readBlock
function reads in a (user-defined) number of TBN frames and returns a 
ObservingBlock object.

For describing the format of data in the file, two function are provided:

getSampleRate
  read in the few frame of an open file handle and return the sampling rate 
  of the data

getFramesPerObs
  read in the first several frames to see how many stands are found in the data.

..versionchanged:: 1.2.0
    Dropped support for ObservingBlock since the lsl.reader.buffer modules does
    a better job.

.. versionchanged:: 0.4.0
    Switched over from pure Python readers to the new C-base Go Fast! readers.

.. versionchanged:: 0.5.0
    Support for ECR 11 TBN header format change.
"""

import copy
import numpy

from lsl.common import dp as dp_common
from lsl.reader._gofast import readTBN
from lsl.reader._gofast import syncError as gsyncError
from lsl.reader._gofast import eofError as geofError
from lsl.reader.errors import baseReaderError, syncError, eofError

__version__ = '0.8'
__revision__ = '$Rev: 2465 $'
__all__ = ['FrameHeader', 'FrameData', 'Frame', 'readFrame', 
           'getSampleRate', 'getFramesPerObs', 'FrameSize', 'filterCodes', 
           '__version__', '__revision__', '__all__']

FrameSize = 1048

# List of filter codes and their corresponding sample rates in Hz
filterCodes = {1:   1000, 2:   3125, 3:    6250, 4:    12500, 5: 25000, 6: 50000, 7: 100000}


class FrameHeader(object):
    """
    Class that stores the information found in the header of a TBW 
    frame.  All three fields listed in the DP ICD version H are stored as 
    well as the original binary header data.
    
    .. versionchanged:: 0.5.0
        Added various attributes to retrieve the central frequnecy 
        and gain that are part of the ECR 11 changes.
    """

    def __init__(self, frameCount=None, tuningWord=None, tbnID=None, gain=None):
        self.frameCount = frameCount
        self.tuningWord = tuningWord
        self.tbnID = tbnID
        self.gain = gain
        self.sampleRate = None
        
    def isTBN(self):
        """
        Function to check if the data is really TBN and not TBW by examining
        the TBN ID field.  Returns True if the data is TBN, false otherwise.
        """

        mode = (self.tbnID>>15)&1
        if mode == 0:
            return True
        else:
            return False

    def parseID(self):
        """
        Function to parse the TBN ID field and return a tuple of the stand 
        number and polarization.
        """

        if self.tbnID&1023 % 2 == 0:
            stand = (self.tbnID&1023) // 2
            pol = 1
        else:
            stand = (self.tbnID&1023) // 2 + 1
            pol = 0
            
        return (stand, pol)

    def getCentralFreq(self):
        """
        Convert the tuning word to a frequency in Hz.
        """

        return dp_common.fS * self.tuningWord / 2**32

    def getGain(self):
        """
        Get the current TBN gain for this frame.
        """

        return self.gain

    def setSampleRate(self, sampleRate):
        """
        Function to set the sample rate of the TBN data in Hz.
        """

        self.sampleRate = sampleRate

    def getFilterCode(self):
        """
        Function to convert the sample rate in Hz to a filter code.
        """
        
        if self.sampleRate is None:
            return None
        else:
            sampleCodes = {}
            for key in filterCodes:
                value = filterCodes[key]
                sampleCodes[value] = key

            return sampleCodes[self.sampleRate]


class FrameData(object):
    """
    Class that stores the information found in the data section of a TBN
    frame.  Both fields listed in the DP ICD version H are stored.
    
    .. versionchanged:: 0.5.0
        Removed various attributes related to storing a central frequnecy 
        and gain that aren't needed with ECR 11.
    """

    def __init__(self, timeTag=None, iq=None):
        self.timeTag = timeTag
        self.iq = iq

    def getTime(self):
        """
        Function to convert the time tag from samples since the UNIX epoch
        (UTC 1970-01-01 00:00:00) to seconds since the UNIX epoch.
        """
        
        return self.timeTag / dp_common.fS


class Frame(object):
    """
    Class that stores the information contained within a single TBN 
    frame.  It's properties are FrameHeader and FrameData objects.
    
    .. versionchanged:: 0.5.0
        Removed various attributes related to storing a central frequnecy 
        and gain that aren't needed with ECR 11.
    """

    def __init__(self, header=None, data=None):
        if header is None:
            self.header = FrameHeader()
        else:
            self.header = header
            
        if data is None:
            self.data = FrameData()
        else:
            self.data = data
            
        self.valid = True
        
    def isTBN(self):
        """
        Convenience wrapper for the Frame.FrameHeader.isTBN function.
        """
        
        return self.header.isTBN()
        
    def parseID(self):
        """
        Convenience wrapper for the Frame.FrameHeader.parseID function.
        """
        
        return self.header.parseID()
        
    def getTime(self):
        """
        Convenience wrapper for the Frame.FrameData.getTime function.
        """
        
        return self.data.getTime()

    def getFilterCode(self):
        """
        Convenience wrapper for the Frame.FrameData.getFilterCode function.
        """

        return self.header.getFilterCode()

    def setSampleRate(self, sampleRate):
        """
        Convenience wrapper for the Frame.FrameData.setSampleRate function.
        """

        self.header.setSampleRate(sampleRate)

    def getCentralFreq(self):
        """
        Convenience wrapper for the Frame.FrameHeader.getCentralFreq function.
        """

        return self.header.getCentralFreq()

    def getGain(self):
        """
        Convenience wrapper for the Frame.FrameHeader.getGain function.
        """

        return self.header.getGain()
            
    def __add__(self, y):
        """
        Add the data sections of two frames together or add a number 
        to every element in the data section.
        """
        
        newFrame = copy.deepcopy(self)
        newFrame += y
        return newFrame
            
    def __iadd__(self, y):
        """
        In-place add the data sections of two frames together or add 
        a number to every element in the data section.
        """
        
        try:
            self.data.iq += y.data.iq
        except AttributeError:
            self.data.iq += numpy.complex64(y)
        return self
        
    def __mul__(self, y):
        """
        Multiple the data sections of two frames together or multiply 
        a number to every element in the data section.
        """
        
        newFrame = copy.deepcopy(self)
        newFrame *= y
        return newFrame
    
    def __imul__(self, y):
        """
        In-place multiple the data sections of two frames together or 
        multiply a number to every element in the data section.
        """
        
        try:
            self.data.iq *= y.data.iq
        except AttributeError:
            self.data.iq *= numpy.complex64(y)
        return self

    def __eq__(self, y):
        """
        Check if the time tags of two frames are equal or if the time
        tag is equal to a particular value.
        """
        
        tX = self.data.timeTag
        try:
            tY = y.data.timeTag
        except AttributeError:
            tY = y
        
        if tX == tY:
            return True
        else:
            return False
            
    def __ne__(self, y):
        """
        Check if the time tags of two frames are not equal or if the time
        tag is not equal to a particular value.
        """
        
        tX = self.data.timeTag
        try:
            tY = y.data.timeTag
        except AttributeError:
            tY = y
        
        if tX != tY:
            return True
        else:
            return False
            
    def __gt__(self, y):
        """
        Check if the time tag of the first frame is greater than that of a
        second frame or if the time tag is greater than a particular value.
        """
        
        tX = self.data.timeTag
        try:
            tY = y.data.timeTag
        except AttributeError:
            tY = y
        
        if tX > tY:
            return True
        else:
            return False
            
    def __ge__(self, y):
        """
        Check if the time tag of the first frame is greater than or equal to 
        that of a second frame or if the time tag is greater than a particular 
        value.
        """
        
        tX = self.data.timeTag
        try:
            tY = y.data.timeTag
        except AttributeError:
            tY = y
        
        if tX >= tY:
            return True
        else:
            return False
            
    def __lt__(self, y):
        """
        Check if the time tag of the first frame is less than that of a
        second frame or if the time tag is greater than a particular value.
        """
        
        tX = self.data.timeTag
        try:
            tY = y.data.timeTag
        except AttributeError:
            tY = y
        
        if tX < tY:
            return True
        else:
            return False
            
    def __le__(self, y):
        """
        Check if the time tag of the first frame is less than or equal to 
        that of a second frame or if the time tag is greater than a particular 
        value.
        """
        
        tX = self.data.timeTag
        try:
            tY = y.data.timeTag
        except AttributeError:
            tY = y
        
        if tX <= tY:
            return True
        else:
            return False
            
    def __cmp__(self, y):
        """
        Compare two frames based on the time tags.  This is helpful for 
        sorting things.
        """
        
        tX = self.data.timeTag
        tY = y.data.timeTag
        if tY > tX:
            return -1
        elif tX > tY:
            return 1
        else:
            return 0


def readFrame(filehandle, SampleRate=None, Verbose=False):
    """
    Function to read in a single TBN frame (header+data) and store the 
    contents as a Frame object.
    """

    # New Go Fast! (TM) method
    try:
        newFrame = readTBN(filehandle, Frame())
    except gsyncError:
        mark = filehandle.tell() - FrameSize
        raise syncError(location=mark)
    except geofError:
        raise eofError
    
    if SampleRate is not None:
        newFrame.setSampleRate(SampleRate)
        
    return newFrame


def getSampleRate(filehandle, nFrames=None, FilterCode=False):
    """
    Find out what the sampling rate/filter code is from consecutive sets of 
    observations.  By default, the rate in Hz is returned.  However, the 
    corresponding filter code can be returned instead by setting the FilterCode
    keyword to True.
    """

    # Save the current position in the file so we can return to that point
    fhStart = filehandle.tell()

    if nFrames is None:
        nFrames = 520
    nFrames = 4*nFrames

    # Build up the list-of-lists that store ID codes and loop through 2,080
    # frames.  In each case, parse pull the TBN ID, extract the stand 
    # number, and append the stand number to the relevant polarization array 
    # if it is not already there.
    frames = {}
    for i in range(nFrames):
        try:
            cFrame = readFrame(filehandle)
        except eofError:
            break
        except syncError:
            continue
        
        stand, pol = cFrame.parseID()
        key = 2*stand + pol
        try:
            frames[key].append(cFrame)
        except:
            frames[key] = [cFrame,]
            
    # Return to the place in the file where we started
    filehandle.seek(fhStart)

    # Any key with complete data will work for this, so pick the first key with two
    # valid frames
    keyCount = 0
    frame1 = None
    frame2 = None
    frameKeys = list(frames.keys())
    while frame1 is None and frame2 is None:
        validKey = frameKeys[keyCount]
        
        try:
            frame1 = frames[validKey][0]
        except IndexError:
            frame1 = None
            
        try:
            frame2 = frames[validKey][1]
        except IndexError:
            frame2 = None

        keyCount = keyCount + 1

    # Now that we have two valid frames that follow one another in time, load in their
    # time tags and calculate the sampling rate.  Since the time tags are based off f_S
    # @ 196 MSPS, and each frame contains 512 samples, the sampling rate is:
    #  f_S / <difference in time tags per 512 samples>
    time1 = frame1.data.timeTag
    time2 = frame2.data.timeTag
    rate = dp_common.fS / (abs( time2 - time1 ) / 512)

    if not FilterCode:
        return rate
    else:
        sampleCodes = {}
        for key in filterCodes:
            value = filterCodes[key]
            sampleCodes[value] = key

        return sampleCodes[rate]


def getFramesPerObs(filehandle):
    """
    Find out how many frames are present per observation by examining 
    the first 2,080 TBN frames.  Return the number of frames per observations 
    as a two-	element tuple, one for each polarization.
    
    So many TBN frames are read in order to try to compensate for the inter-
    leaving of the packets from the various DP1 boards during the recording.
    
    .. note::
        Post-IOC it is probably simpler to adopt a value of the number of 
        frames per observation of 520 rather than try to find it from the
        file.
    """
    
    # Save the current position in the file so we can return to that point
    fhStart = filehandle.tell()

    # Build up the list-of-lists that store ID codes and loop through 600
    # frames.  In each case, parse pull the TBN ID, extract the stand 
    # number, and append the stand number to the relevant polarization array 
    # if it is not already there.
    idCodes = [[], []]
    maxX = 0
    maxY = 0
    for i in range(4*520):
        try:
            cFrame = readFrame(filehandle)
        except eofError:
            break
        except syncError:
            continue
        
        cID, cPol = cFrame.header.parseID()
        if cID not in idCodes[cPol]:
            idCodes[cPol].append(cID)
        
        # Also, look at the actual IDs to try to figure out how many of
        # each there are in case the frames are out of order.  Will this
        # help in all cases?  Probably not but we should at least try it
        if cPol == 0:
            if cID > maxX:
                maxX = cID
        else:
            if cID > maxY:
                maxY = cID
            
    # Return to the place in the file where we started
    filehandle.seek(fhStart)
    
    # Compare idCodes sizes with maxX and maxY.  Load maxX and maxY with 
    # the larger of the two values.
    if maxX < len(idCodes[0]):
        maxX = len(idCodes[0])
    if maxY < len(idCodes[1]):
        maxY = len(idCodes[1])
    
    # Get the length of each beam list and return them as a tuple
    return (maxX, maxY)
